/*
 * @(#)RandomAdversary.java
 *
 * $Revision: 1.3 $
 * $Date: 2006/04/06 19:10:12 $
 *
 * Copyright (c) 2005-2006 RWTH Aachen University, Lehr- und Forschungsgebiet Informatik 4.
 * All rights reserved.
 */

package de.rwth.informatik.lufgi4.trustedpals.adversary;

import java.util.Date;
import java.util.Random;

import de.rwth.informatik.lufgi4.trustedpals.messaging.Message;

/**
 * This adversary randomly (with a given probability) accepts or drops a given message.
 * 
 * @author Milan Fort
 * @version $Revision: 1.3 $, $Date: 2006/04/06 19:10:12 $
 * 
 * @since 1.0
 */
public class RandomAdversary implements Adversary {

    private Random random = new Random(new Date().getTime());
    
    private int probability = 0;
    
    /**
     * 
     * @param probability acceptance probability of messages
     */
    public RandomAdversary(int probability) {
        if (probability > 100 || probability < 0) {
            throw new IllegalArgumentException("Probability must be between 0 and 100: " + probability);
        }
        
        this.probability = probability;
    }
    
    public boolean accept(Message message) {
        return random.nextInt(100) < probability;
    }
}
