/*
 * @(#)Controller.java
 *
 * $Revision: 1.6 $
 * $Date: 2006/04/06 19:10:12 $
 *
 * Copyright (c) 2005-2006 RWTH Aachen University, Lehr- und Forschungsgebiet Informatik 4.
 * All rights reserved.
 */

package de.rwth.informatik.lufgi4.trustedpals.controlling;

import java.util.Arrays;
import java.util.Iterator;
import java.util.List;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import de.rwth.informatik.lufgi4.trustedpals.adversary.Adversary;
import de.rwth.informatik.lufgi4.trustedpals.messaging.Message;
import de.rwth.informatik.lufgi4.trustedpals.messaging.MessageReceiver;
import de.rwth.informatik.lufgi4.trustedpals.messaging.MessageSender;
import de.rwth.informatik.lufgi4.trustedpals.protocol.Protocol;
import de.rwth.informatik.lufgi4.trustedpals.protocol.Item;
import de.rwth.informatik.lufgi4.trustedpals.util.Utils;

/**
 * 
 * 
 * @author Milan Fort
 * @version $Revision: 1.6 $, $Date: 2006/04/06 19:10:12 $
 * 
 * @since 1.0
 */
public final class Controller {
    
    private static Log logger = LogFactory.getLog(Controller.class);
    
    private int hostId;
    
    private byte[] wantedItemDescription;
    
    private Protocol protocol;
    
    private Adversary incomingAdversary;
    
    private Adversary outgoingAdversary;
    
    private MessageSender messageSender;
    
    private MessageReceiver messageReceiver;

    public Controller() {}
    
    public void startProtocol() {
        System.out.println("----==[Starting Protocol]==----");
        logger.info("Starting protocol");
        
        protocol.start();
    }
    
    private void sendMessages() {
        List messages = protocol.getMessages();
        
        if (messages.size() == 0) {
            logger.debug("No messages were created in this round");
            
        } else {
            logger.debug("Sending out " + messages.size() + " message(s)");
        }
        
        for (Iterator i = messages.iterator(); i.hasNext();) {
            Message message = (Message) i.next();
            
            if (outgoingAdversary.accept(message)) {
                messageSender.sendMessage(message);
                
            } else {
                logger.warn("Outgoing adversary has dropped the message: " + message);
            }
        }

    }
    
    public void startSendPhase() {
        System.out.println("----==[Starting Send Phase]==----");
        logger.debug("Starting send phase");
        
        if (protocol.hasErrorOccured()) {
            logger.error("Unknown error occured during the calculation run");
            System.exit(-1);
        }

        if (protocol.isFinished()) {
            System.out.println("----==[Protocol Finished]==----");
            logger.info("Protocol finished");
            
            sendMessages();
            
            Item receivedItem = protocol.getItem();
            logger.debug("Received item: " + receivedItem);
            
            if (Arrays.equals(wantedItemDescription, receivedItem.getDescription())) {
                logger.info("I have received the wanted item");
                
            } else {
                logger.error("Received item is not the one I wanted");
            }
            
            System.exit(0);
            
        } else if (protocol.isComputationFinished()) {
            logger.debug("Computation phase has finished within alloted time");
            
            sendMessages();
            
        } else {
            logger.error("Computation phase has not finished within allotted time");
            System.exit(-1);
        }
    }
    
    public void startNextRound() {
        System.out.println("----==[Starting Next Round]==----");
        logger.debug("Starting next round");
        
        List messages = messageReceiver.getReceivedMessages();
        
        for (Iterator i = messages.iterator(); i.hasNext();) {
            Message message = (Message) i.next();
            
            if (incomingAdversary.accept(message)) {
                logger.debug("Processing message: " + message);
                
            } else {
                logger.warn("Incoming adversary has dropped the message: " + message);
                i.remove();
            }
        }
        
        protocol.startNextRound(messages);
        messageReceiver.clearReceivedMessages();
    }
    
    /* Properties */
    
    public int getHostId() {
        return hostId;
    }
    
    public void setHostId(int hostId) {
        this.hostId = hostId;
    }

    public void setWantedItemDescription(byte[] wantedItemDescription) {
        this.wantedItemDescription = wantedItemDescription;
        
        //TODO: temporary solution to overcome configuration conversion problems
        Utils.convertNumbers(this.wantedItemDescription);
    }
    
    public Protocol getProtocol() {
        return protocol;
    }
    
    public void setProtocol(Protocol calculation) {
        this.protocol = calculation;
    }

    public Adversary getIncomingAdversary() {
        return incomingAdversary;
    }
    
    public void setIncomingAdversary(Adversary incomingAdversary) {
        this.incomingAdversary = incomingAdversary;
    }
    
    public Adversary getOutgoingAdversary() {
        return outgoingAdversary;
    }
    
    public void setOutgoingAdversary(Adversary outgoingAdversary) {
        this.outgoingAdversary = outgoingAdversary;
    }

    public void setMessageSender(MessageSender messageSender) {
        this.messageSender = messageSender;
    }
    
    public void setMessageReceiver(MessageReceiver messageReceiver) {
        this.messageReceiver = messageReceiver;
    }
}
