/*
 * @(#)CardListener.java
 *
 * $Revision: 1.3 $
 * $Date: 2006/04/06 19:10:12 $
 *
 * Copyright (c) 2005-2006 RWTH Aachen University, Lehr- und Forschungsgebiet Informatik 4.
 * All rights reserved.
 */

package de.rwth.informatik.lufgi4.trustedpals.gui;

import java.util.Enumeration;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import opencard.core.service.SmartCard;
import opencard.core.service.CardRequest;
import opencard.core.terminal.CardTerminal;
import opencard.core.terminal.CardTerminalException;
import opencard.core.terminal.CardTerminalRegistry;
import opencard.core.event.CTListener;
import opencard.core.event.CardTerminalEvent;
import opencard.core.event.EventGenerator;

/**
 * This class encapsulates the communication with smartcards using the OCF framework.
 * It initializes the OCF framework and informs the controller about events from the card reader.
 * It is responsible for closing the OCF framework as well. 
 * 
 * @author Milan Fort
 * @version $Revision: 1.3 $, $Date: 2006/04/06 19:10:12 $
 * 
 * @since 1.0
 */
final class CardListener implements CTListener {
    
    private static Log logger = LogFactory.getLog(CardListener.class);
    
    private SmartCard smartCard = null;

    private CardTerminal terminal = null;
    
    private int slotID = 0;

    private GUIController controller = null;
    
    CardListener(GUIController controller) throws CardListenerException {
        this.controller = controller;
        
        try {
            /* Starting the OCF framework */
            SmartCard.start();

            /* Dummy code to remove nonexistent terminal references from the registry. */
            CardTerminalRegistry ctr = CardTerminalRegistry.getRegistry();
            Enumeration terminals = ctr.getCardTerminals();
            ctr.remove((CardTerminal) terminals.nextElement());
            ctr.remove((CardTerminal) terminals.nextElement());
            
            register();
            
        } catch (Exception e) {
            try {
                SmartCard.shutdown();
                
            } catch (CardTerminalException cte) {
                // ignore...
            }
            
            throw new CardListenerException(e);
        }
    }
    
    private void register() throws CardTerminalException {
        EventGenerator.getGenerator().addCTListener(this);
        EventGenerator.getGenerator().createEventsForPresentCards(this);
    }
    
    private void unregister() {
        EventGenerator.getGenerator().removeCTListener(this);
    }
    
    void close() {
        try {
            unregister();
            SmartCard.shutdown();
            
        } catch (CardTerminalException e) {
            logger.error("Error closing OCF framework: " + e.getMessage());
        }
    }
    
    public void cardInserted(CardTerminalEvent event) {
        if (smartCard == null) {
            try {
                smartCard = SmartCard.getSmartCard(event,
                        new CardRequest(CardRequest.ANYCARD, null, null));
                
                terminal = event.getCardTerminal();
                slotID = event.getSlotID();
                
//                byte[] atr = terminal.getCardID(slotID).getATR();
                controller.cardInserted(smartCard);
                
            } catch (CardTerminalException e) {
                logger.error("Communication with the smartcard failed");
            }
        }
    }
    
    public void cardRemoved(CardTerminalEvent event) {
        if ((event.getSlotID() == slotID) && (event.getCardTerminal() == terminal)) {
            smartCard = null;
            terminal = null;
            slotID = 0;
            controller.cardRemoved();
        }
    }
}
