/*
 * @(#)ConfigurationParser.java
 *
 * $Revision: 1.3 $
 * $Date: 2006/04/06 19:10:12 $
 *
 * Copyright (c) 2005-2006 RWTH Aachen University, Lehr- und Forschungsgebiet Informatik 4.
 * All rights reserved.
 */

package de.rwth.informatik.lufgi4.trustedpals.gui;

/**
 * 
 * 
 * @author Milan Fort
 * @version $Revision: 1.3 $, $Date: 2006/04/06 19:10:12 $
 * 
 * @since 1.0
 */
final class ConfigurationParser {
    
    byte hostCount;
    
    byte hostID;
    
    byte sourceHost;
    
    byte destinationHost;
    
    byte[] ownItemDescription;
    
    byte[] ownItemContent;
    
    byte[] wantedItemDescription;
    
    byte[] seed;
    
    ConfigurationParser(byte[] data, boolean hasSeed) {
        byte currentPossition = 0;
        
        hostCount = data[currentPossition++];
        
        hostID = data[currentPossition++];
        
        sourceHost = data[currentPossition++];
        
        destinationHost = data[currentPossition++];
        
        byte ownItemDescriptionLength = data[currentPossition++];
        ownItemDescription = new byte[ownItemDescriptionLength];
        System.arraycopy(data, currentPossition, ownItemDescription, (byte) 0, ownItemDescriptionLength);

        currentPossition += ownItemDescriptionLength;
        
        byte ownItemContentLength = data[currentPossition++];
        ownItemContent = new byte[ownItemContentLength];
        System.arraycopy(data, currentPossition, ownItemContent, (byte) 0, ownItemContentLength);
        
        currentPossition += ownItemContentLength;
        
        byte wantedItemDescriptionLength = data[currentPossition++];
        wantedItemDescription = new byte[wantedItemDescriptionLength];
        System.arraycopy(data, currentPossition, wantedItemDescription, (byte) 0,
                wantedItemDescriptionLength);
        
        currentPossition += wantedItemDescriptionLength;
        
        if (hasSeed) {
            byte seedLength = data[currentPossition++];
            seed = new byte[seedLength];
            System.arraycopy(data, currentPossition, seed, (byte) 0, seedLength);
            
        } else {
            seed = new byte[0];
        }
    }
    
    static String byteArrayToString(byte[] x) {
        StringBuffer sb = new StringBuffer();
        
        for (int i = 0; i < x.length; i++) {
            sb.append(x[i]);
        }
        
        return sb.toString();
    }

    static void checkNumber(byte b) {
        if (b <= 0 || b > 9) {
            throw new NumberFormatException("Each number must be between 1 and 9");
        }
    }
    
    byte getHostCount() {
        return hostCount;
    }

    byte getHostID() {
        return hostID;
    }

    byte getSourceHost() {
        return sourceHost;
    }

    byte getDestinationHost() {
        return destinationHost;
    }

    byte[] getOwnItemDescription() {
        return ownItemDescription;
    }

    String getOwnItemDescriptionAsString() {
        return byteArrayToString(ownItemDescription);
    }
    
    byte[] getOwnItemContent() {
        return ownItemContent;
    }

    String getOwnItemContentAsString() {
        return byteArrayToString(ownItemContent);
    }
    
    byte[] getWantedItemDescription() {
        return wantedItemDescription;
    }

    String getWantedItemDescriptionAsString() {
        return byteArrayToString(wantedItemDescription);
    }

    byte[] getSeed() {
        return seed;
    }
    
    String getSeedAsString() {
        return byteArrayToString(seed);
    }
}
