/*
 * @(#)GDController.java
 *
 * $Revision: 1.3 $
 * $Date: 2006/04/06 19:10:12 $
 *
 * Copyright (c) 2005-2006 RWTH Aachen University, Lehr- und Forschungsgebiet Informatik 4.
 * All rights reserved.
 */

package de.rwth.informatik.lufgi4.trustedpals.gui;

import org.apache.commons.lang.ArrayUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import opencard.core.service.SmartCard;
import opencard.core.terminal.CardTerminalException;
import opencard.core.terminal.CommandAPDU;
import opencard.core.terminal.ResponseAPDU;
import opencard.opt.util.PassThruCardService;

/**
 * 
 * 
 * @author Milan Fort
 * @version $Revision: 1.3 $, $Date: 2006/04/06 19:10:12 $
 * 
 * @since 1.0
 */
public class GDController implements GUIController {
    
    private static Log logger = LogFactory.getLog(GDController.class);

    private static final byte FAIR_EXCHANGE_CLA = (byte) 0xC0;
    
    private static final byte INIT_INS = (byte) 0x20;
    
    private static final byte GET_CONFIGURATION_INS = (byte) 0x30;

    private static final byte P1 = (byte) 0x00;
    
    private static final byte P2 = (byte) 0x00;
    
    private static final byte EMPTY = (byte) 0x00;

    private static final byte OFFSET_CDATA = 5;
    
    // Applet ID: A0 00 00 00 62 03 01 0C 05 01
    // 00 A4 04 00 0A A0 00 00 00 62 03 01 0C 05 01
    private CommandAPDU selectFairExchangeApplet = new CommandAPDU(new byte[] {
            (byte) 0x00, (byte) 0xA4, (byte) 0x04, (byte) 0x00, (byte) 0x0A,  (byte) 0xA0,
            (byte) 0x00, (byte) 0x00, (byte) 0x00, (byte) 0x62, (byte) 0x03,  (byte) 0x01,
            (byte) 0x0C, (byte) 0x05, (byte) 0x01, (byte) 0x7F
    });
    
    private CommandAPDU getConfigurationAPDU = new CommandAPDU(new byte[] {
            FAIR_EXCHANGE_CLA, GET_CONFIGURATION_INS, P1, P2, (byte) 0x00, (byte) 0x02});

    private PassThruCardService card = null;

    private View view = null;
    
    private CardListener cardListener = null;
    
    public GDController() {
        view = new View(this);
        
        view.display();
        view.hideSeed();
        view.markCardNotPresent();
        
        try {
            cardListener = new CardListener(this);
            
        } catch (CardListenerException e) {
            logger.error("Unable to connect to smartcard interface", e);
            System.exit(1);
        }
    }
    
    public void handleSendData() {
        try {
            byte hostCount = Byte.parseByte(view.getHostCount());
            byte hostID = Byte.parseByte(view.getHostID());
            byte sourceHost = Byte.parseByte(view.getSourceHost());
            byte destinationHost = Byte.parseByte(view.getDestinationHost());

            ConfigurationParser.checkNumber(hostCount);
            ConfigurationParser.checkNumber(hostID);
            ConfigurationParser.checkNumber(sourceHost);
            ConfigurationParser.checkNumber(destinationHost);
            
            String ownItemDescriptionString = view.getOwnItemDescription();
            byte[] ownItemDescription = new byte[ownItemDescriptionString.length()];
            
            for (int i = 0; i < ownItemDescription.length; i++) {
                ownItemDescription[i] = Byte.parseByte("" + ownItemDescriptionString.charAt(i));
            }

            String ownItemContentString = view.getOwnItemContent();
            byte[] ownItemContent = new byte[ownItemContentString.length()];
            
            for (int i = 0; i < ownItemContent.length; i++) {
                ownItemContent[i] = Byte.parseByte("" + ownItemContentString.charAt(i));
            }

            String wantedItemDescriptionString = view.getWantedItemDescription();
            byte[] wantedItemDescription = new byte[wantedItemDescriptionString.length()];
            
            for (int i = 0; i < wantedItemDescription.length; i++) {
                wantedItemDescription[i] = Byte.parseByte("" + wantedItemDescriptionString.charAt(i));
            }
            
            byte[] data = createDataArray(hostCount, hostID, sourceHost, destinationHost,
                    ownItemDescription, ownItemContent, wantedItemDescription);

            logger.debug("Sending data: " + ArrayUtils.toString(data));
            
            CommandAPDU command = new CommandAPDU(new byte[] {
                    FAIR_EXCHANGE_CLA, INIT_INS, P1, P2, (byte) data.length,
                    EMPTY, EMPTY, EMPTY, EMPTY, EMPTY, EMPTY, EMPTY, EMPTY, EMPTY, EMPTY, 
                    EMPTY, EMPTY, EMPTY, EMPTY, EMPTY, EMPTY, EMPTY, EMPTY, EMPTY, EMPTY,
                    EMPTY, EMPTY, EMPTY, EMPTY, EMPTY, EMPTY, EMPTY, EMPTY, EMPTY, EMPTY});
            
            byte[] buffer = command.getBuffer();
            System.arraycopy(data, 0, buffer, OFFSET_CDATA, data.length);
            
            logger.debug("APDU buffer is: " + ArrayUtils.toString(buffer));
            
            ResponseAPDU response = card.sendCommandAPDU(command);
            System.out.println("Send response: 0x" + Integer.toHexString(response.sw()));
            
            view.clearFields();
            
            //TODO: read the new data from the card and set it again
            view.setHostCount(hostCount);
            view.setHostID(hostID);
            view.setSourceHost(sourceHost);
            view.setDestinationHost(destinationHost);
            view.setOwnItemDescription(ConfigurationParser.byteArrayToString(ownItemDescription));
            view.setOwnItemContent(ConfigurationParser.byteArrayToString(ownItemContent));
            view.setWantedItemDescription(ConfigurationParser.byteArrayToString(wantedItemDescription));
            
        } catch (NumberFormatException e) {
            view.showErrorMessage();
            return;

        } catch (CardTerminalException e) {
            logger.error("Error occured in communication with the smartcard", e);
            return;
        }
    }
    
    static byte[] createDataArray(byte hostCount, byte hostID, byte sourceHost, byte destinationHost,
            byte[] ownItemDescription, byte[] ownItemContent, byte[] wantedItemDescription)
    {
        byte[] data = new byte[7 + ownItemDescription.length + ownItemContent.length
                               + wantedItemDescription.length];
        
        data[0] = hostCount;
        data[1] = hostID;
        data[2] = sourceHost;
        data[3] = destinationHost;

        data[4] = (byte) ownItemDescription.length;
        System.arraycopy(ownItemDescription, 0, data, 5, ownItemDescription.length);
        
        data[5 + ownItemDescription.length] = (byte) ownItemContent.length;
        System.arraycopy(ownItemContent, 0, data, 6 + ownItemDescription.length,
                ownItemContent.length);
        
        data[6 + ownItemDescription.length + ownItemContent.length] =
            	(byte) wantedItemDescription.length;
        
        System.arraycopy(wantedItemDescription, 0, data,
                7 + ownItemDescription.length + ownItemContent.length, wantedItemDescription.length);
        
        return data;
    }
    
    public void cardInserted(SmartCard smartCard) {
        logger.debug("Card inserted");

        view.markCardInserted();
        
        try {
            card = (PassThruCardService) smartCard.getCardService(PassThruCardService.class, true);
            
            ResponseAPDU response = card.sendCommandAPDU(selectFairExchangeApplet);
            logger.debug("Selecting Fair Exchange Applet: 0x" + Integer.toHexString(response.sw()));
            
            response = card.sendCommandAPDU(getConfigurationAPDU);
            logger.debug("Getting configuration data: 0x" + Integer.toHexString(response.sw()));
            byte[] data = response.data();
            
            logger.debug("Retrieved data: " + ArrayUtils.toString(data));
            
            if ((response.sw1() & 0xFF) == 0x90 && response.sw2() == 0x00) {
                ConfigurationParser parser = new ConfigurationParser(data, false);
                
                view.setHostCount(parser.getHostCount());
                view.setHostID(parser.getHostID());
                view.setSourceHost(parser.getSourceHost());
                view.setDestinationHost(parser.getDestinationHost());
                
                view.setOwnItemDescription(parser.getOwnItemDescriptionAsString());
                view.setOwnItemContent(parser.getOwnItemContentAsString());
                view.setWantedItemDescription(parser.getWantedItemDescriptionAsString());
            }
            
        } catch (Exception e) {
            logger.error("Error in communication with the smartcard", e);
        }
    }
    
    public void cardRemoved() {
        view.markCardNotPresent();
    }
    
    public void shutdown() {
        cardListener.close();
        
        System.exit(0);
    }
    
    /** Self-testing module. */
    public static void main(String[] args) {
        new GDController();
    }
}
