/*
 * @(#)View.java
 *
 * $Revision: 1.4 $
 * $Date: 2006/04/06 19:10:12 $
 *
 * Copyright (c) 2005-2006 RWTH Aachen University, Lehr- und Forschungsgebiet Informatik 4.
 * All rights reserved.
 */

package de.rwth.informatik.lufgi4.trustedpals.gui;

import java.awt.*;
import java.awt.event.*;

import javax.swing.*;

/**
 * 
 * 
 * @author Milan Fort
 * @version $Revision: 1.4 $, $Date: 2006/04/06 19:10:12 $
 * 
 * @since 1.0
 */
final class View {

    private static final String CARD_NOT_PRESENT_STATUS =
        	"<html><font color='#CC0000'>CARD NOT PRESENT</font></html>";

    private static final String CARD_INSERTED_STATUS =
    		"<html><font color='#00C000'>CARD INSERTED</font></html>";

    private JFrame frame;

    private JLabel cardStatus;
    
    private JLabel seedLabel;
    
    private JTextField hostCount;

    private JTextField hostID;
    
    private JTextField sourceHost;
    
    private JTextField destinationHost;
    
    private JTextField ownItemDescription;
    
    private JTextField ownItemContent;
    
    private JTextField wantedItemDescription;
    
    private JTextField seed;
    
    private JButton sendButton;
    
    private JButton exitButton;
    
    View(final GUIController controller) {
        //JFrame.setDefaultLookAndFeelDecorated(true);
        
        frame = new JFrame("TrustedPals Configuration Console");
        frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        frame.setSize(new Dimension(400, 295));
        frame.setResizable(false);
        
        Dimension d = Toolkit.getDefaultToolkit().getScreenSize();
        int locationX = (int) ((d.getWidth() - frame.getWidth()) / 2);
        int locationY = (int) ((d.getHeight() - frame.getHeight()) / 2);
        frame.setLocation(locationX, locationY);
        
        JPanel inputPanel = new JPanel(new SpringLayout());

        JLabel label = new JLabel("Host Count:", JLabel.TRAILING);
        label.setDisplayedMnemonic(KeyEvent.VK_H);
        hostCount = new JTextField(20);
        hostCount.setFont(new Font("Helvetica", Font.BOLD, 12));
        label.setLabelFor(hostCount);
        inputPanel.add(label);
        inputPanel.add(hostCount);

        label = new JLabel("Host ID:", JLabel.TRAILING);
        label.setDisplayedMnemonic(KeyEvent.VK_I);
        hostID = new JTextField(20);
        hostID.setFont(new Font("Helvetica", Font.BOLD, 12));
        label.setLabelFor(hostID);
        inputPanel.add(label);
        inputPanel.add(hostID);

        label = new JLabel("Source Host:", JLabel.TRAILING);
        label.setDisplayedMnemonic(KeyEvent.VK_O);
        sourceHost = new JTextField(20);
        sourceHost.setFont(new Font("Helvetica", Font.BOLD, 12));
        label.setLabelFor(sourceHost);
        inputPanel.add(label);
        inputPanel.add(sourceHost);

        label = new JLabel("Destination Host:", JLabel.TRAILING);
        label.setDisplayedMnemonic(KeyEvent.VK_E);
        destinationHost = new JTextField(20);
        destinationHost.setFont(new Font("Helvetica", Font.BOLD, 12));
        label.setLabelFor(destinationHost);
        inputPanel.add(label);
        inputPanel.add(destinationHost);
        
        label = new JLabel("Own Item Description:", JLabel.TRAILING);
        label.setDisplayedMnemonic(KeyEvent.VK_N);
        ownItemDescription = new JTextField(20);
        ownItemDescription.setFont(new Font("Helvetica", Font.BOLD, 12));
        label.setLabelFor(ownItemDescription);
        inputPanel.add(label);
        inputPanel.add(ownItemDescription);

        label = new JLabel("Own Item Content:", JLabel.TRAILING);
        label.setDisplayedMnemonic(KeyEvent.VK_C);
        ownItemContent = new JTextField(20);
        ownItemContent.setFont(new Font("Helvetica", Font.BOLD, 12));
        label.setLabelFor(ownItemContent);
        inputPanel.add(label);
        inputPanel.add(ownItemContent);
        
        label = new JLabel("Wanted Item Description:", JLabel.TRAILING);
        label.setDisplayedMnemonic(KeyEvent.VK_W);
        wantedItemDescription = new JTextField(20);
        wantedItemDescription.setFont(new Font("Helvetica", Font.BOLD, 12));
        label.setLabelFor(wantedItemDescription);
        inputPanel.add(label);
        inputPanel.add(wantedItemDescription);

        label = new JLabel("Seed:", JLabel.TRAILING);
        label.setDisplayedMnemonic(KeyEvent.VK_D);
        seed = new JTextField(20);
        seed.setFont(new Font("Helvetica", Font.BOLD, 12));
        label.setLabelFor(seed);
        inputPanel.add(label);
        inputPanel.add(seed);
        seedLabel = label;

        SpringUtilities.makeCompactGrid(inputPanel, 8, 2, //rows, cols
                10, 10, //initX, initY
                10, 6); //xPad, yPad

        sendButton = new JButton("Send");
        sendButton.setMnemonic(KeyEvent.VK_S);
        sendButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                controller.handleSendData();
            }
        });

        exitButton = new JButton("Exit");
        exitButton.setMnemonic(KeyEvent.VK_X);
        exitButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                controller.shutdown();
            }
        });

        exitButton.setPreferredSize(sendButton.getPreferredSize());

        cardStatus = new JLabel();
        cardStatus.setHorizontalAlignment(SwingConstants.CENTER);

        JPanel buttonPanel = new JPanel();
        buttonPanel.setBorder(BorderFactory.createEmptyBorder(20, 10, 10, 10));
        buttonPanel.setLayout(new BoxLayout(buttonPanel, BoxLayout.LINE_AXIS));
        
        buttonPanel.add(sendButton);
//        buttonPanel.add(Box.createHorizontalGlue());
        buttonPanel.add(cardStatus);
//        buttonPanel.add(Box.createHorizontalGlue());
        buttonPanel.add(exitButton);
        
        JPanel contentPane = new JPanel(new BorderLayout());
        contentPane.add(inputPanel, BorderLayout.NORTH);
        contentPane.add(buttonPanel, BorderLayout.SOUTH);
        
        //content panes must be opaque
        contentPane.setOpaque(true);
        frame.setContentPane(contentPane);
        
        //frame.getRootPane().setDefaultButton(sendButton);

        frame.addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent e) {
                controller.shutdown();
            }
        });
        
        markCardNotPresent();
        
//        new Thread() {
//            public void run() {
//                try {
//                    Thread.sleep(5000);
//                } catch (InterruptedException e) {
//                    // ignore...
//                }
//                markCardInserted();
//                //showErrorMessage();
//            }
//        }.start();
    }

    String getHostCount() {
        return hostCount.getText();
    }
    
    void setHostCount(int hostCount) {
        this.hostCount.setText("" + hostCount);
    }
    
    String getHostID() {
        return hostID.getText();
    }
    
    void setHostID(int hostID) {
        this.hostID.setText("" + hostID);
    }
    
    String getSourceHost() {
        return sourceHost.getText();
    }
    
    void setSourceHost(int sourceHost) {
        this.sourceHost.setText("" + sourceHost);
    }

    String getDestinationHost() {
        return destinationHost.getText();
    }
    
    void setDestinationHost(int destinationHost) {
        this.destinationHost.setText("" + destinationHost);
    }
    
    String getOwnItemDescription() {
        return ownItemDescription.getText();
    }
    
    void setOwnItemDescription(String ownItemDescription) {
        this.ownItemDescription.setText(ownItemDescription);
    }
    
    String getOwnItemContent() {
        return ownItemContent.getText();
    }
    
    void setOwnItemContent(String ownItemContent) {
        this.ownItemContent.setText(ownItemContent);
    }
    
    String getWantedItemDescription() {
        return wantedItemDescription.getText();
    }
    
    void setWantedItemDescription(String wantedItemDescription) {
        this.wantedItemDescription.setText(wantedItemDescription);
    }

    String getSeed() {
        return seed.getText();
    }
    
    void setSeed(String seed) {
        this.seed.setText(seed);
    }
    
    void clearFields() {
        hostCount.setText("");
        hostID.setText("");
        sourceHost.setText("");
        destinationHost.setText("");
        ownItemDescription.setText("");
        ownItemContent.setText("");
        wantedItemDescription.setText("");
        seed.setText("");
    }

    void showErrorMessage() {
        JOptionPane optionPane = new JOptionPane("Invalid data!", JOptionPane.ERROR_MESSAGE);
            
        JDialog optionPaneDialog = optionPane.createDialog(frame, "Error");
        
        optionPaneDialog.setVisible(true);
    }
    
    void markCardNotPresent() {
        cardStatus.setText(CARD_NOT_PRESENT_STATUS);
        hide();
    }
    
    void markCardInserted() {
        cardStatus.setText(CARD_INSERTED_STATUS);
        show();
    }
    
    void hideSeed() {
        seedLabel.setVisible(false);
        seed.setVisible(false);
    }
    
    private void hide() {
        clearFields();
        
        hostCount.setEditable(false);
        hostID.setEditable(false);
        sourceHost.setEditable(false);
        destinationHost.setEditable(false);
        ownItemDescription.setEditable(false);
        ownItemContent.setEditable(false);
        wantedItemDescription.setEditable(false);
        seed.setEditable(false);
        
        sendButton.setEnabled(false);
        //exitButton.setEnabled(false);
    }
    
    private void show() {
        clearFields();
        
        hostCount.setEditable(true);
        hostID.setEditable(true);
        sourceHost.setEditable(true);
        destinationHost.setEditable(true);
        ownItemDescription.setEditable(true);
        ownItemContent.setEditable(true);
        wantedItemDescription.setEditable(true);
        seed.setEditable(true);
        
        sendButton.setEnabled(true);
        exitButton.setEnabled(true);
    }

    void display() {
        javax.swing.SwingUtilities.invokeLater(new Runnable() {
            public void run() {
                //frame.pack();
                frame.setVisible(true);
            }
        });
    }
    
	/** Self-testing module. */
    public static void main(String[] args) {
        View view = new View(null);
        view.display();
   }
}
