/*
 * @(#)MessageReceiver.java
 *
 * $Revision: 1.5 $
 * $Date: 2006/04/06 19:10:12 $
 *
 * Copyright (c) 2005-2006 RWTH Aachen University, Lehr- und Forschungsgebiet Informatik 4.
 * All rights reserved.
 */

package de.rwth.informatik.lufgi4.trustedpals.messaging;

import java.util.ArrayList;
import java.util.List;

import javax.jms.JMSException;
import javax.jms.MessageListener;
import javax.jms.ObjectMessage;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

/**
 * This class is a Message-Driven POJO; it is used for receiving messages via JMS.
 * 
 * @author Milan Fort
 * @version $Revision: 1.5 $, $Date: 2006/04/06 19:10:12 $
 * 
 * @since 1.0
 */
public final class MessageReceiver implements MessageListener {
    
    private static Log logger = LogFactory.getLog(MessageReceiver.class);

    private int hostId = 0;
    
    private List storedMessages;
    
    public MessageReceiver() {
        storedMessages = new ArrayList();
    }
    
    public void setHostId(int hostId) {
        this.hostId = hostId;
    }
    
    public void onMessage(javax.jms.Message jmsMessage) {
        if (jmsMessage instanceof ObjectMessage) {
            try {
                Message message = (Message) ((ObjectMessage) jmsMessage).getObject();
                
                if (message.getReceiver() != hostId) {
                    return;
                }
                
                synchronized(this) {
                    storedMessages.add(message);
                }
                logger.debug("Received message: " + message);
                
            } catch (JMSException e) {
                logger.error("Unable to retrieve the content of an object message", e);
            }
            
        } else {
            logger.error("Unknown message received: " + jmsMessage);
        }
    }
    
    public synchronized void clearReceivedMessages() {
        storedMessages.clear();
    }
    
    public synchronized List getReceivedMessages() {
        return new ArrayList(storedMessages);
    }
}
