/*
 * @(#)GDFairExchangeApplet.java
 *
 * $Revision: 1.2 $
 * $Date: 2006/04/06 19:10:12 $
 *
 * Copyright (c) 2005-2006 RWTH Aachen University, Lehr- und Forschungsgebiet Informatik 4.
 * All rights reserved.
 */

package de.rwth.informatik.lufgi4.trustedpals.protocol;

import javacard.framework.*;

/**
 * 
 * 
 * @author Milan Fort
 * @version $Revision: 1.2 $, $Date: 2006/04/06 19:10:12 $
 * 
 * @since 1.0
 */
public final class GDFairExchangeApplet extends Applet {
    
    /* Constants */
    
    /** Code of CLA byte in the command APDU header */
    private static final byte FAIR_EXCHANGE_CLA = (byte) 0xC0;

    /* Codes of INS byte in the command APDU header */
    
    private static final byte INIT_INS = (byte) 0x20;
    
    private static final byte GET_CONFIGURATION_INS = (byte) 0x30;
    
    private static final byte START_INS = (byte) 0x40;
    
    private static final byte START_NEXT_ROUND_INS = (byte) 0x50;
    
    private static final byte FINISHED_QUERY_INS = (byte) 0x60;

    private static final byte GET_MESSAGES_INS = (byte) 0x70;
    
    private static final byte GET_OUTPUT_INS = (byte) 0x80;
    
    /* Applet-specific status words */
    
    private static final short SW_PROTOCOL_ALREADY_FINISHED = 0x6A85;
    
    /* Restrictions */
    
    private static final byte MAX_SUPPORTED_HOSTS = 10;
    
    private static final short MAX_SUPPORTED_DATA_LENGTH =
        (MAX_SUPPORTED_HOSTS - 1) * (5 + (2 * MAX_SUPPORTED_HOSTS)) + 1;
    
    private static final byte MAX_ITEM_DESCRIPTION_LENGTH = 7;
    
    private static final byte MAX_ITEM_CONTENT_LENGTH = 10;

    /* Message constants */
    
    private static final byte MESSAGE_OFFSET_SENDER = 0;
    
    private static final byte MESSAGE_OFFSET_RECEIVER = 1;
    
    private static final byte MESSAGE_OFFSET_HOST_COUNT = 2;
    
    private static final byte MESSAGE_OFFSET_ROUND = 3;
    
    private static final byte MESSAGE_OFFSET_RANDOM_NUMBER = 4;
    
    private static final byte MESSAGE_OFFSET_ITEM = 5;
    
    private static final byte MESSAGE_OFFSET_FLAGS = 4;
    
    private static final byte MESSAGE_OFFSET_VOTE = 5;
    
    private static final byte MESSAGE_FLAG_NORMAL = 1;
    
    private static final byte MESSAGE_FLAG_FAKE = 2;
    
    private static final byte MESSAGE_FLAG_DUMMY = 3;
    
    /* Configurable attributes */
    
    private byte hostID = 0;
    
    private byte hostCount = 0;
    
    private byte sourceHost = 0;
    
    private byte destinationHost = 0;
    
    private byte ownItemDescriptionLength = 0;
    
    private byte[] ownItemDescription = null;

    private byte ownItemContentLength = 0;
    
    private byte[] ownItemContent = null;
    
    // Note: Difference between applet version and test version!
    private byte wantedItemDescriptionLength = 0;
    
    private byte[] wantedItemDescription = null;

    /* Other attributes */
    
    private boolean finished = false;
    
    private boolean shouldReturnReceivedItem = false;
    
    private byte receivedItemDescriptionLength = 0;
    
    private byte[] receivedItemDescription = null;

    private byte receivedItemContentLength = 0;
    
    private byte[] receivedItemContent = null;
    
    private HostSet vote = null;
    
    private HostSet newVote = null;
    
    private HostSet locked = null;
    
    private HostSet suspected = null;
    
    private byte[] receivedVote = null;
    
    private byte[] receivedLocked = null;
    
    /** Data received for the current's round computation. */
    private byte[] data = null;
    
    /** Messages to be sent out during the next sending phase. */
    private byte[] messages = null;
    
    /** Current round. */
    private byte round = 0;
    
    /** Random number k. */
    private byte fakeRoundsCount = 0;

    /** Allowed maximum of faulty processes. */
    private byte maxFaultyProcesses = 0;

    private byte[] temp = null;
    
    private GDFairExchangeApplet() {
        data = new byte[MAX_SUPPORTED_DATA_LENGTH];
        messages = new byte[MAX_SUPPORTED_DATA_LENGTH];
        
        ownItemDescription = new byte[MAX_ITEM_DESCRIPTION_LENGTH];
        ownItemContent = new byte[MAX_ITEM_CONTENT_LENGTH];

        wantedItemDescription = new byte[MAX_ITEM_DESCRIPTION_LENGTH];
        
        receivedItemDescription = new byte[MAX_ITEM_DESCRIPTION_LENGTH];
        receivedItemContent = new byte[MAX_ITEM_CONTENT_LENGTH];
        
        vote = new HostSet(MAX_SUPPORTED_HOSTS);
        newVote = new HostSet(MAX_SUPPORTED_HOSTS);
        locked = new HostSet(MAX_SUPPORTED_HOSTS);
        suspected = new HostSet(MAX_SUPPORTED_HOSTS);
        
        receivedVote = new byte[MAX_SUPPORTED_HOSTS];
        receivedLocked = new byte[MAX_SUPPORTED_HOSTS];
        
        temp = new byte[MAX_SUPPORTED_HOSTS];
        
        register();
    }

    public static void install(byte[] bArray, short bOffset, byte bLength) {
        new GDFairExchangeApplet();
    }

    public void process(APDU apdu) {

        // APDU object carries a byte array (buffer) to
        // transfer incoming and outgoing APDU header
        // and data bytes between card and CAD

        // At this point, only the first header bytes
        // [CLA, INS, P1, P2, P3] are available in
        // the APDU buffer.
        // The interface javacard.framework.ISO7816
        // declares constants to denote the offset of
        // these bytes in the APDU buffer

        byte[] buffer = apdu.getBuffer();
        
        // check SELECT APDU command
        if ((buffer[ISO7816.OFFSET_CLA] == 0) && (buffer[ISO7816.OFFSET_INS] == (byte)(0xA4))) {
            return;
        }

//        // Or like this
//        if (selectingApplet()) {
//            return;
//        }
        
        if (buffer[ISO7816.OFFSET_CLA] != FAIR_EXCHANGE_CLA) {
            ISOException.throwIt(ISO7816.SW_CLA_NOT_SUPPORTED);
        }

        switch (buffer[ISO7816.OFFSET_INS]) {
        case INIT_INS:
        	initialize(apdu);
        	return;
        	
        case GET_CONFIGURATION_INS:
        	getConfiguration(apdu);
        	return;

        case START_INS:
        	start();
        	return;

        case START_NEXT_ROUND_INS:
        	startNextRound(apdu);
        	return;

        case FINISHED_QUERY_INS:
        	finishedQuery(apdu);
        	return;

        case GET_MESSAGES_INS:
        	getMessages(apdu);
        	return;

        case GET_OUTPUT_INS:
        	getOutput(apdu);
        	return;
        	
        default:
        	ISOException.throwIt(ISO7816.SW_INS_NOT_SUPPORTED);
        }
    }

    private void initialize(APDU apdu) {
        byte[] buffer = apdu.getBuffer();
        
        byte lc = buffer[ISO7816.OFFSET_LC];
        
        byte byteRead = (byte) apdu.setIncomingAndReceive();
        
        if (lc != byteRead) {
            ISOException.throwIt(ISO7816.SW_WRONG_LENGTH);
        }

        byte currentPossition = ISO7816.OFFSET_CDATA;
        
        hostCount = buffer[currentPossition++];
        if (hostCount > MAX_SUPPORTED_HOSTS || hostCount <= 0) {
            ISOException.throwIt(ISO7816.SW_WRONG_DATA);
        }
        
        hostID = buffer[currentPossition++];
        if (hostID > hostCount || hostID <= 0) {
            ISOException.throwIt(ISO7816.SW_WRONG_DATA);
        }
        
        sourceHost = buffer[currentPossition++];
        if (sourceHost > hostCount || sourceHost <= 0 || sourceHost == hostID) {
            ISOException.throwIt(ISO7816.SW_WRONG_DATA);
        }
        
        destinationHost = buffer[currentPossition++];
        if (destinationHost > hostCount || destinationHost <= 0 || destinationHost == hostID) {
            ISOException.throwIt(ISO7816.SW_WRONG_DATA);
        }
        
        ownItemDescriptionLength = buffer[currentPossition++];
        if (ownItemDescriptionLength > MAX_ITEM_DESCRIPTION_LENGTH || ownItemDescriptionLength <= 0) {
            ISOException.throwIt(ISO7816.SW_WRONG_DATA);
        }
        
        Util.arrayCopyNonAtomic(buffer, currentPossition, ownItemDescription, (byte) 0,
                ownItemDescriptionLength);

        currentPossition += ownItemDescriptionLength;
        
        ownItemContentLength = buffer[currentPossition++];
        if (ownItemContentLength > MAX_ITEM_CONTENT_LENGTH || ownItemContentLength <= 0) {
            ISOException.throwIt(ISO7816.SW_WRONG_DATA);
        }
        
        Util.arrayCopyNonAtomic(buffer, currentPossition, ownItemContent, (byte) 0,
                ownItemContentLength);
        
        currentPossition += ownItemContentLength;
        
        wantedItemDescriptionLength = buffer[currentPossition++];
        if (wantedItemDescriptionLength > MAX_ITEM_DESCRIPTION_LENGTH ||
                wantedItemDescriptionLength <= 0) {
            
            ISOException.throwIt(ISO7816.SW_WRONG_DATA);
        }
        
        Util.arrayCopyNonAtomic(buffer, currentPossition, wantedItemDescription, (byte) 0,
                wantedItemDescriptionLength);
        
        maxFaultyProcesses = computeMaxFaultyProcesses();
        
//        logger.debug("Maximum faulty processes (t) is: " + maxFaultyProcesses);
    }
    
    /*
     * Note: the invocation of this method should be protected by PIN
     * or some other security mechanism. 
     */
    private void getConfiguration(APDU apdu) {
        byte[] buffer = apdu.getBuffer();
        
        short le = apdu.setOutgoing();
        
        short totalBytes =
            (short) (7 + ownItemDescriptionLength + ownItemContentLength + wantedItemDescriptionLength);
        
        apdu.setOutgoingLength(totalBytes);
        
        buffer[0] = hostCount;
        buffer[1] = hostID;
        buffer[2] = sourceHost;
        buffer[3] = destinationHost;
        buffer[4] = ownItemDescriptionLength;
        buffer[5] = ownItemContentLength;
        buffer[6] = wantedItemDescriptionLength;
        
        // first 5 bytes from the apdu buffer
        apdu.sendBytes((short) 0, (short) 5);
        
        // own item description
        apdu.sendBytesLong(ownItemDescription, (short) 0, (short) ownItemDescriptionLength);
        
        // own item content length from the apdu buffer
        apdu.sendBytes((short) 5, (short) 1);
        
        // own item content
        apdu.sendBytesLong(ownItemContent, (short) 0, (short) ownItemContentLength);

        // wanted item description length from the apdu buffer
        apdu.sendBytes((short) 6, (short) 1);
        
        // wanted item description
        apdu.sendBytesLong(wantedItemDescription, (short) 0, (short) wantedItemDescriptionLength);
    }
    
    private void start() {
        finished = false;
        shouldReturnReceivedItem = false;
        
        round = 0;
        clearData();
        clearMessages();
        clearReceivedItem();
        clearReceivedVoteAndLocked();
        vote.clear();
        newVote.clear();
        locked.clear();
        suspected.clear();

//        logger.info("Starting round " + round);
        
        if (isInitiator()) {
//            logger.trace("I am the Initiator");
            
            byte k = pickRandomNumber();
            
//            logger.debug("Generated random number k=" + k);
            
            fakeRoundsCount = k;
            
            prepareInitiatorMessages();
            
        } else {
//            logger.trace("I am NOT the Initiator");
            
            // to be received from initiator later
            fakeRoundsCount = 0;
            
            prepareMessageWithItem();
        }
    }
    
    private void startNextRound(APDU apdu) {
        if (finished) {
            ISOException.throwIt(SW_PROTOCOL_ALREADY_FINISHED);
        }
        
        byte[] buffer = apdu.getBuffer();
        
        byte lc = buffer[ISO7816.OFFSET_LC];
        
        byte byteRead = (byte) apdu.setIncomingAndReceive();
        
        if (lc != byteRead) {
            ISOException.throwIt(ISO7816.SW_WRONG_LENGTH);
        }

        Util.arrayCopyNonAtomic(buffer, (byte) ISO7816.OFFSET_CDATA, data, (byte) 0, byteRead);
        
        round++;
        clearMessages();
        
//        logger.info("Starting round " + round);
        
        if (round == 1) {
            if (hasReceivedItemFromSourceHost()) {
//                logger.debug("Received an item from source host");
                
                if (isReceivedItemCorrect()) {
//                    logger.debug("Received item is correct");
                    
                } else {
//                    logger.error("Received item is NOT correct");
                    
                    terminateProtocolWithError();
                    return;
                }
                
            } else {
//                logger.error("Did not receive the item from source host");
                
                terminateProtocolWithError();
                return;
            }
            
            if (!isInitiator()) {
                if (hasReceivedRandomNumberFromInitiator()) {
//                    logger.debug("Received fake rounds count: " + fakeRoundsCount);
                    
                } else {
//                    logger.error("Did not receive fake rounds count");
                    
                    terminateProtocolWithError();
                    return;
                }
            }
            
            prepareFakeMessages();
            
        } else if (round <= fakeRoundsCount) {
            //logger.info("This round is just fake");
            
            if (hasReceivedFakeMessagesFromAllHosts()) {
//                logger.info("Fake messages received from all hosts");
                
            } else {
//                logger.error("Did not receive fake messages from all other hosts");
                
                terminateProtocolWithError();
                return;
            }
            
            prepareFakeMessages();
            
        } else if (round == fakeRoundsCount + 1) {
            if (hasReceivedFakeMessagesFromAllHosts()) {
//                logger.info("Fake messages received from all hosts");
                
            } else {
//                loggser.error("Did not receive fake messages from all hosts");
                
                terminateProtocolWithError();
                return;
            }
            
            //do not prepare fake messages anymore
//            logger.info("Real calculation is starting now");
            
            vote.add((byte) hostID);
            newVote.add((byte) hostID);
            locked.clear();
            suspected.clear();
            
            prepareNormalMessages();
            
        } else if (round <= maxFaultyProcesses + fakeRoundsCount + 2) {
            newVote.clear();
            
            byte[] voteArray = vote.toArray();
            byte[] newVoteArray = newVote.toArray();
            
            for (byte i = 1; i <= hostCount; i++) {

                // We do not send messages to ourselves, so ignore this case - is this a problem?
                if (i != hostID) {
                    if (!suspected.contains(i)) {
                        if (hasReceivedNormalMessageFromHost(i)) {
//                            logger.debug("Received a normal messaged from host " + i);
                            
                            for (byte j = 0; j < hostCount; j++) {
                                if (receivedVote[j] == HostSet.TRUE && voteArray[j] == HostSet.FALSE) {
                                    voteArray[j] = HostSet.TRUE;
                                    newVoteArray[j] = HostSet.TRUE;
                                    locked.addAll(receivedLocked);
                                }
                            }
                        }
                        
                    } else if (!locked.contains(i)) {
//                        logger.warn("Did not receive a normal message from host " + i);
                        
                        suspected.add(i);
                    }
                }
            }
            
            if (suspected.size() > maxFaultyProcesses) {
//                logger.error("Suspecting already at least half of all hosts");
                
                terminateProtocolWithError();
                return;
            }

            if (!locked.contains((byte) hostID)) {
                
                if ((round - (fakeRoundsCount + 1) > suspected.size()) || !locked.isEmpty()) {
                    locked.add((byte) hostID);
                }
                
            } else if (locked.size() > maxFaultyProcesses) {
                decide();
                return;
            }
            
            if (round <= maxFaultyProcesses + fakeRoundsCount + 1) {
                prepareNormalMessages();
                
            } else {
//                assert round == maxFaultyProcesses + fakeRoundsCount + 2;
                
                decide();
                return;
            }
            
        } else {
//            assert false;
        }
    }
    
    private void finishedQuery(APDU apdu) {
        byte[] buffer = apdu.getBuffer();

        short le = apdu.setOutgoing();

//        if ( le < 2 ) ISOException.throwIt(ISO7816.SW_WRONG_LENGTH);

        apdu.setOutgoingLength((byte) 1);

        buffer[0] = (byte) (finished ? 1 : 0); 

        apdu.sendBytes((short) 0, (short) 1);
    }
    
    private void getMessages(APDU apdu) {
        short le = apdu.setOutgoing();
        
        apdu.setOutgoingLength((short) messages.length);
        
        apdu.sendBytesLong(messages, (short) 0, (short) messages.length);
    }
    
    private void getOutput(APDU apdu) {
        byte[] buffer = apdu.getBuffer();
        
        if (finished && shouldReturnReceivedItem) {
            short le = apdu.setOutgoing();
            
            short totalBytes = (short) (receivedItemDescriptionLength + receivedItemContentLength + 2);
            
            apdu.setOutgoingLength(totalBytes);

            buffer[0] = receivedItemDescriptionLength;
            buffer[1] = receivedItemContentLength;

            // description length
            apdu.sendBytes((short) 0, (short) 1);
            
            // description
            apdu.sendBytesLong(receivedItemDescription, (short) 0, (short) receivedItemDescriptionLength);
            
            // content length
            apdu.sendBytes((short) 1, (short) 1);
            
            // content
            apdu.sendBytesLong(receivedItemContent, (short) 0, (short) receivedItemContentLength);
            
        } else {
            buffer[0] = 0;
            apdu.setOutgoingAndSend((short) 0, (short) 1);
        }
    }
    
    private void fill(byte[] x, byte value) {
        Util.arrayFillNonAtomic(x, (short) 0, (short) x.length, value);
    }
    
    private void clear(byte[] x) {
        fill(x, (byte) 0);
    }
    
    private void clearMessages() {
        fill(messages, (byte) 0);
    }
    
    private void clearData() {
        fill(data, (byte) 0);
    }
    
    private void clearReceivedItem() {
        receivedItemDescriptionLength = 0;
        receivedItemContentLength = 0;
        
        fill(receivedItemDescription, (byte) 0);
        fill(receivedItemContent, (byte) 0);
    }
    
    private void clearReceivedVoteAndLocked() {
        fill(receivedVote, (byte) 0);
        fill(receivedLocked, (byte) 0);
    }

    private boolean isInitiator() {
        return hostID == 1;
    }
    
    private byte pickRandomNumber() {
//        byte randomNumber = (byte) (Math.random() * 10);
//        
//        randomNumber %= 4;
//        randomNumber++;
//        
//        return randomNumber;
        
        // Not supported yet
        return 2;
    }
    
    private byte computeMaxFaultyProcesses() {
        return (byte) (hostCount % 2 == 0 ? hostCount / 2 - 1 : hostCount / 2);
    }
    
    /**
     * 
     * 
     * @return the length of the just prepared message
     */
    private byte prepareMessageWithItem() {
//        logger.debug("Preparing message with item for host " + destinationHost);
        
        if (isInitiator()) {
//            logger.debug("Random number k=" + fakeRoundsCount + " will be also included");
        }
        
//        assert round == 0;
        clearMessages();

        messages[MESSAGE_OFFSET_SENDER] = (byte) hostID;
        messages[MESSAGE_OFFSET_RECEIVER] = (byte) destinationHost;
        messages[MESSAGE_OFFSET_HOST_COUNT] = (byte) hostCount;
        messages[MESSAGE_OFFSET_ROUND] = (byte) round;
        messages[MESSAGE_OFFSET_RANDOM_NUMBER] = (byte) fakeRoundsCount;
        
        messages[MESSAGE_OFFSET_ITEM] = ownItemDescriptionLength;
        Util.arrayCopyNonAtomic(ownItemDescription, (short) 0, messages,
                (short) (MESSAGE_OFFSET_ITEM + 1), ownItemDescriptionLength);
        
        messages[MESSAGE_OFFSET_ITEM + ownItemDescriptionLength + 1] = ownItemContentLength;
        Util.arrayCopyNonAtomic(ownItemContent, (short) 0, messages,
                (short) (MESSAGE_OFFSET_ITEM + ownItemDescriptionLength + 2), ownItemContentLength);
        
        return  (byte) (MESSAGE_OFFSET_ITEM + ownItemDescriptionLength + ownItemContentLength + 2);
    }
    
    private boolean hasReceivedItemFromSourceHost() {
        clearReceivedItem();
        
        byte currentPosition = 0;
        
        while (currentPosition < data.length) {
            byte sender = data[currentPosition + MESSAGE_OFFSET_SENDER];
            if (sender == 0) {
                break;
            }
            
            if (sender == sourceHost) {
                receivedItemDescriptionLength = data[currentPosition + MESSAGE_OFFSET_ITEM];
                
                if (receivedItemDescriptionLength == 0) {
                    return false;
                }
                
                Util.arrayCopyNonAtomic(data, (short) (currentPosition + MESSAGE_OFFSET_ITEM + 1),
                        receivedItemDescription, (short) 0, receivedItemDescriptionLength);
                
                receivedItemContentLength = data[currentPosition + MESSAGE_OFFSET_ITEM
                                                 + receivedItemDescriptionLength + 1];
                
                Util.arrayCopyNonAtomic(data, (short) (currentPosition + MESSAGE_OFFSET_ITEM
                        + receivedItemDescriptionLength + 2), receivedItemContent, (short) 0,
                        receivedItemContentLength);
                
                return true;
                
            } else {
                byte descriptionLength = data[currentPosition + MESSAGE_OFFSET_ITEM];
                
                if (descriptionLength == 0) {
                    currentPosition += MESSAGE_OFFSET_ITEM + 1;
                    
                } else {
//                    assert descriptionLength > 0;
                    
                    byte itemContentLength = data[currentPosition + MESSAGE_OFFSET_ITEM
                                                     + descriptionLength + 1];
                    
                    currentPosition += MESSAGE_OFFSET_ITEM + descriptionLength + itemContentLength + 2;
                }
            }
        }
        
        return false;
    }
    
    private boolean isReceivedItemCorrect() {
        
        if (receivedItemDescriptionLength == wantedItemDescriptionLength) {
            for (byte i = 0; i < receivedItemDescriptionLength; i++) {
                if (receivedItemDescription[i] != wantedItemDescription[i]) {
                    return false;
                }
            }
            
            for (byte i = receivedItemDescriptionLength; i < receivedItemDescription.length; i++) {
                if (receivedItemDescription[i] != 0) {
                    return false;
                }
            }
            
        } else {
            return false;
        }
        
        return true;
    }
    
    private void prepareInitiatorMessages() {
//        assert round == 0;
        clearMessages();
        
        byte currentPosition = prepareMessageWithItem();
        
        for (byte i = 1; i <= hostCount; i++) {
            if (i != hostID && i != destinationHost) {
//                logger.debug("Preparing message with random number k=" + fakeRoundsCount
//                        + " for host " + i);
                
                messages[currentPosition + MESSAGE_OFFSET_SENDER] = (byte) hostID;
                messages[currentPosition + MESSAGE_OFFSET_RECEIVER] = (byte) i;
                messages[currentPosition + MESSAGE_OFFSET_HOST_COUNT] = (byte) hostCount;
                messages[currentPosition + MESSAGE_OFFSET_ROUND] = (byte) round;
                messages[currentPosition + MESSAGE_OFFSET_RANDOM_NUMBER] = (byte) fakeRoundsCount;
                messages[currentPosition + MESSAGE_OFFSET_ITEM] = (byte) 0x0;
                
                currentPosition += 6;
            }
        }
    }
    
    private boolean hasReceivedRandomNumberFromInitiator() {
        if (hostID == 1) {
            return true;
        }
        
        byte currentPosition = 0;
        
        while (currentPosition < data.length) {
            byte sender = data[currentPosition + MESSAGE_OFFSET_SENDER];
            if (sender == 0) {
                break;
            }
            
            if (sender == 1) {
                fakeRoundsCount = data[currentPosition + MESSAGE_OFFSET_RANDOM_NUMBER]; 
                
                return true;
                
            } else {
                byte descriptionLength = data[currentPosition + MESSAGE_OFFSET_ITEM];
                
                if (descriptionLength == 0) {
                    currentPosition += MESSAGE_OFFSET_ITEM + 1;
                    
                } else {
//                    assert descriptionLength > 0;
                    
                    byte itemContentLength = data[currentPosition + MESSAGE_OFFSET_ITEM
                                                     + descriptionLength + 1];
                    
                    currentPosition += MESSAGE_OFFSET_ITEM + descriptionLength + itemContentLength + 2;
                }
            }
        }
        
        return false;
    }
    
    private void prepareFakeMessages() {
//        assert round > 0;
        clearMessages();
        
        byte currentPosition = 0;
        
        for (byte i = 1; i <= hostCount; i++) {
            if (i != hostID) {
                messages[currentPosition + MESSAGE_OFFSET_SENDER] = (byte) hostID;
                messages[currentPosition + MESSAGE_OFFSET_RECEIVER] = (byte) i;
                messages[currentPosition + MESSAGE_OFFSET_HOST_COUNT] = (byte) hostCount;
                messages[currentPosition + MESSAGE_OFFSET_ROUND] = (byte) round;
                messages[currentPosition + MESSAGE_OFFSET_FLAGS] = MESSAGE_FLAG_FAKE;
                
                currentPosition += normalMessageLength();
            }
        }
    }
    
    private boolean hasReceivedFakeMessagesFromAllHosts() {
        clear(temp);
        temp[hostID-1] = 1;
        
        byte currentPosition = 0;
        
        while (currentPosition < data.length) {
            byte sender = data[currentPosition + MESSAGE_OFFSET_SENDER];
            if (sender == 0) {
                break;
            }
            
            temp[sender-1] = 1;
            
            currentPosition += normalMessageLength();
        }
        
        for (byte i = 0; i < hostCount; i++) {
            if (temp[i] != 1) {
                return false;
            }
        }
        
        return true;
    }
    
    private void prepareNormalMessages() {
//        assert round > 0;
        clearMessages();
        
        byte currentPosition = 0;
        
        for (byte i = 1; i <= hostCount; i++) {
            if (i != hostID) {
                messages[currentPosition + MESSAGE_OFFSET_SENDER] = (byte) hostID;
                messages[currentPosition + MESSAGE_OFFSET_RECEIVER] = i;
                messages[currentPosition + MESSAGE_OFFSET_HOST_COUNT] = (byte) hostCount;
                messages[currentPosition + MESSAGE_OFFSET_ROUND] = (byte) round;
                
                if (suspected.contains(i)) {
                    messages[currentPosition + MESSAGE_OFFSET_FLAGS] = MESSAGE_FLAG_DUMMY;
                    
                } else {
                    messages[currentPosition + MESSAGE_OFFSET_FLAGS] = MESSAGE_FLAG_NORMAL;
                }
                
                Util.arrayCopyNonAtomic(newVote.toArray(), (short) 0, messages,
                        (short) (currentPosition + MESSAGE_OFFSET_VOTE), hostCount);
                
                Util.arrayCopyNonAtomic(locked.toArray(), (short) 0, messages,
                        (short) (currentPosition + MESSAGE_OFFSET_VOTE + hostCount), hostCount);
                
                currentPosition += normalMessageLength();
            }
        }
    }
    
    private boolean hasReceivedNormalMessageFromHost(byte senderHostID) {
        clearReceivedVoteAndLocked();
        byte currentPosition = 0;
        
        while (currentPosition < data.length) {
            byte sender = data[currentPosition + MESSAGE_OFFSET_SENDER];
            if (sender == 0) {
                break;
            }
            
            if (sender == senderHostID) {
                if (data[currentPosition + MESSAGE_OFFSET_FLAGS] == MESSAGE_FLAG_NORMAL) {
                    Util.arrayCopyNonAtomic(data, (short) (currentPosition + MESSAGE_OFFSET_VOTE),
                            receivedVote, (short) 0, hostCount);
                    
                    Util.arrayCopyNonAtomic(data,
                            (short) (currentPosition + MESSAGE_OFFSET_VOTE + hostCount), receivedLocked,
                            (short) 0, hostCount);
                
                    return true;
                    
                } else {
                    return false;
                }
            }
            
            currentPosition += normalMessageLength();
        }
        
        return false;
    }
    
    private void terminateProtocolWithError() {
        shouldReturnReceivedItem = false;
        finished = true;
    }
    
    private void decide() {
        shouldReturnReceivedItem = true;
        
        for (byte i = 1; i <= hostCount; i++) {
            if (!vote.contains(i)) {
                shouldReturnReceivedItem = false;
                break;
            }
        }

        finished = true;
    }
    
    private byte normalMessageLength() {
        return (byte) (2 * hostCount + 5);
    }
}
