/*
 * @(#)Item.java
 *
 * $Revision: 1.5 $
 * $Date: 2006/04/06 19:10:12 $
 *
 * Copyright (c) 2005-2006 RWTH Aachen University, Lehr- und Forschungsgebiet Informatik 4.
 * All rights reserved.
 */

package de.rwth.informatik.lufgi4.trustedpals.protocol;

import java.io.Serializable;

import org.apache.commons.lang.builder.EqualsBuilder;
import org.apache.commons.lang.builder.HashCodeBuilder;
import org.apache.commons.lang.builder.ToStringBuilder;

import de.rwth.informatik.lufgi4.trustedpals.util.Utils;

/**
 * Item to be exchanged between communicating parties.
 * 
 * @author Milan Fort
 * @version $Revision: 1.5 $, $Date: 2006/04/06 19:10:12 $
 * 
 * @since 1.0
 */
public final class Item implements Serializable {
    
    private static final long serialVersionUID = 8351L;

    private byte[] description = null;
    
    private byte[] content = null;
    
    public Item() {
        this(new byte[0], new byte[0]);
    }
    
    public Item(byte[] description, byte[] content) {
        this.description = description;
        this.content = content;
        
        //TODO: temporary solution to overcome configuration conversion problems
        Utils.convertNumbers(this.description);
        Utils.convertNumbers(this.content);
    }
    
    public byte[] getDescription() {
        return description;
    }
    
    public byte[] getContent() {
        return content;
    }
    
    public void destroy() {
        description = new byte[0];
        content = new byte[0];
    }
    
    public boolean equals(Object obj) {
        return EqualsBuilder.reflectionEquals(this, obj);
    }
    
    public int hashCode() {
        return HashCodeBuilder.reflectionHashCode(13, 43, this);
    }
    
    public String toString() {
        return ToStringBuilder.reflectionToString(this, Utils.SHORT_STYLE);
    }
    
    /** Self-testing module. */
    public static void main(String[] args) {
        Item item = new Item(new byte[] {1, 2, 3}, new byte[] {5, 4, 3, 2, 1});

        System.out.println(item);
    }
}
