/*
 * @(#)JavaCardRandom.java
 *
 * $Revision: 1.2 $
 * $Date: 2006/04/06 19:10:12 $
 *
 * Copyright (c) 2005-2006 RWTH Aachen University, Lehr- und Forschungsgebiet Informatik 4.
 * All rights reserved.
 */

package de.rwth.informatik.lufgi4.trustedpals.random;

import javacard.framework.ISOException;
import javacard.framework.Util;
import javacard.security.MessageDigest;

/**
 * A Pseudo-random number generator implemented using a secure hash function. 
 * 
 * @author Milan Fort
 * @version $Revision: 1.2 $, $Date: 2006/04/06 19:10:12 $
 * 
 * @since 1.0
 */
public final class JavaCardRandom {
    
    private static final short SW_WRONG_SEED_LENGTH = 0x7A00;
    
    private static final short SW_INVALID_SEED = 0x7A01;
    
    private static final short MAX_SEED_LENGTH = 10;
    
    private static final short HASH_LENGTH = 20;
    
    private static final byte HASH_ALGORITHM = MessageDigest.ALG_SHA;
    
    private short seedLength = 0;
    
    private short seedByteToUpdate = 0;
    
    private byte[] seed = null;
    
    private byte[] hash = null;
    
    MessageDigest messageDigest = null;

    public JavaCardRandom() {
        seedLength = 0;
        seedByteToUpdate = 0;
        seed = new byte[MAX_SEED_LENGTH];
        hash = new byte[HASH_LENGTH];
        
        messageDigest = MessageDigest.getInstance(HASH_ALGORITHM, false);
    }

    public void setSeed(byte[] buffer, short offset, short length) {
        Util.arrayFillNonAtomic(seed, (short) 0, (short) seed.length, (byte) 0);
        
        if (length > MAX_SEED_LENGTH || length <= 0) {
            ISOException.throwIt(SW_WRONG_SEED_LENGTH);
        }
        
        seedLength = length;
        Util.arrayCopyNonAtomic(buffer, offset, seed, (short) 0, length);
        
        seedByteToUpdate = 0;
    }
    
    public byte nextRandomNumber() {
        messageDigest.doFinal(seed, (short) 0, seedLength, hash, (short) 0);
            
        updateSeed();
        
        return hasOddOnesCount(hash) ? (byte) 1 : (byte) 0;
    }
    
    private void updateSeed() {
        seed[seedByteToUpdate] = increaseByte(seed[seedByteToUpdate]);
        
        increaseSeedByteToUpdate();
    }

    private void increaseSeedByteToUpdate() {
        if (seedLength <= 0) {
            ISOException.throwIt(SW_INVALID_SEED);
        }
        
        seedByteToUpdate = (short) ((seedByteToUpdate + 1) % seedLength);
    }
    
    private static byte increaseByte(byte x) {
        return (byte) (x == 127 ? -128 : x + 1);
    }
    
    private static byte computeOnesCount(byte x) {
        byte onesCount = 0;

        for (short i = 1; i <= 128; i *= 2) {
            onesCount += (x & i) == i ? 1 : 0;
        }
        
        return onesCount;
    }
    
    private static boolean hasOddOnesCount(byte[] x) {
        short onesCount = 0;
        
        for (short i = 0; i < x.length; i++) {
            onesCount += computeOnesCount(x[i]);
        }
        
        return onesCount % 2 == 1;
    }
}
