/*
 * @(#)RandomApplet.java
 *
 * $Revision: 1.2 $
 * $Date: 2006/04/06 19:10:12 $
 *
 * Copyright (c) 2005-2006 RWTH Aachen University, Lehr- und Forschungsgebiet Informatik 4.
 * All rights reserved.
 */

package de.rwth.informatik.lufgi4.trustedpals.random;

import javacard.framework.*;

/**
 * 
 * 
 * @author Milan Fort
 * @version $Revision: 1.2 $, $Date: 2006/04/06 19:10:12 $
 * 
 * @since 1.0
 */
public final class RandomApplet extends Applet {
    
    /** Code of CLA byte in the command APDU header */
    private static final byte RANDOM_CLA = (byte) 0xC1;

    /* Codes of INS byte in the command APDU header */
    
    private static final byte SET_SEED_INS = (byte) 0x20;
    
    private static final byte GET_NEXT_RANDOM_NUMBER_INS = (byte) 0x30;

    private JavaCardRandom random = null;
    
    private RandomApplet() {
        random = new JavaCardRandom();
        
        register();
    }

    public static void install(byte[] bArray, short bOffset, byte bLength) {
        new RandomApplet();
    }

    public void process(APDU apdu) {

        // APDU object carries a byte array (buffer) to
        // transfer incoming and outgoing APDU header
        // and data bytes between card and CAD

        // At this point, only the first header bytes
        // [CLA, INS, P1, P2, P3] are available in
        // the APDU buffer.
        // The interface javacard.framework.ISO7816
        // declares constants to denote the offset of
        // these bytes in the APDU buffer

        byte[] buffer = apdu.getBuffer();
        
        // check SELECT APDU command
        if ((buffer[ISO7816.OFFSET_CLA] == 0) && (buffer[ISO7816.OFFSET_INS] == (byte)(0xA4))) {
            return;
        }

//        // Or like this
//        if (selectingApplet()) {
//            return;
//        }
        
        if (buffer[ISO7816.OFFSET_CLA] != RANDOM_CLA) {
            ISOException.throwIt(ISO7816.SW_CLA_NOT_SUPPORTED);
        }

        switch (buffer[ISO7816.OFFSET_INS]) {
        case SET_SEED_INS:
        	setSeed(apdu);
        	return;
        	
        case GET_NEXT_RANDOM_NUMBER_INS:
        	getNextRandomNumber(apdu);
        	return;

        default:
        	ISOException.throwIt(ISO7816.SW_INS_NOT_SUPPORTED);
        }
    }

    private void setSeed(APDU apdu) {
        byte[] buffer = apdu.getBuffer();
        
        byte lc = buffer[ISO7816.OFFSET_LC];
        
        byte byteRead = (byte) apdu.setIncomingAndReceive();
        
        if (lc != byteRead) {
            ISOException.throwIt(ISO7816.SW_WRONG_LENGTH);
        }

        byte currentPossition = ISO7816.OFFSET_CDATA;
        
        byte seedLength = buffer[currentPossition++];
        
        random.setSeed(buffer, currentPossition, seedLength);
    }
    
    private void getNextRandomNumber(APDU apdu) {
        byte[] buffer = apdu.getBuffer();
        
        buffer[0] = random.nextRandomNumber();
        
        apdu.setOutgoingAndSend((short) 0, (short) 1);
    }
}
