/*
 * @(#)RandomAppletMain.java
 *
 * $Revision: 1.2 $
 * $Date: 2006/04/06 19:10:12 $
 *
 * Copyright (c) 2005-2006 RWTH Aachen University, Lehr- und Forschungsgebiet Informatik 4.
 * All rights reserved.
 */

package de.rwth.informatik.lufgi4.trustedpals.random;

import java.util.Enumeration;

import opencard.core.service.CardRequest;
import opencard.core.service.SmartCard;
import opencard.core.terminal.CardTerminal;
import opencard.core.terminal.CardTerminalRegistry;
import opencard.core.terminal.CommandAPDU;
import opencard.core.terminal.ResponseAPDU;
import opencard.opt.util.PassThruCardService;

import org.apache.commons.lang.ArrayUtils;

/**
 * 
 * 
 * @author Milan Fort
 * @version $Revision: 1.2 $, $Date: 2006/04/06 19:10:12 $
 * 
 * @since 1.0
 */
public final class RandomAppletMain {
    
    private static final byte RANDOM_CLA = (byte) 0xC1;
    
    private static final byte SET_SEED_INS = (byte) 0x20;
    
    private static final byte GET_NEXT_RANDOM_NUMBER_INS = (byte) 0x30;

    private static final byte P1 = (byte) 0x00;
    
    private static final byte P2 = (byte) 0x00;
    
    private static final byte EMPTY = (byte) 0x00;

    private static final byte OFFSET_CDATA = 5;
    
    // Applet ID: A0 00 00 00 62 03 01 0B 08 01
    // 00 A4 04 00 0A A0 00 00 00 62 03 01 0B 08 01
    private static CommandAPDU selectRandomApplet = new CommandAPDU(new byte[] {
            (byte) 0x00, (byte) 0xA4, (byte) 0x04, (byte) 0x00, (byte) 0x0A,  (byte) 0xA0,
            (byte) 0x00, (byte) 0x00, (byte) 0x00, (byte) 0x62, (byte) 0x03,  (byte) 0x01,
            (byte) 0x0B, (byte) 0x08, (byte) 0x01, (byte) 0x7F
    });
    
    private static CommandAPDU getNextRandomNumberAPDU = new CommandAPDU(new byte[] {
            RANDOM_CLA, GET_NEXT_RANDOM_NUMBER_INS, P1, P2, (byte) 0x00, (byte) 0x01});

    private static byte[] stringToByteArray(String str) {
        byte[] result = new byte[str.length()];
        
        for (int i = 0; i < str.length(); i++) {
            char c = str.charAt(i);
            
            if (c < '0' || c > '9') {
                throw new NumberFormatException("This character is not a number: " + c);
            }
            
            result[i] = (byte) (c - 48);
        }
     
        return result;
    }

    public static void main(String[] args) throws Exception {
        if (args.length != 2) {
            System.out.println("Invalid parameter list");
            
            System.exit(-1);
        }
        
        byte[] seed = stringToByteArray(args[0]);
        int numbersToGenerate = Integer.parseInt(args[1]);
        int zeroCount = 0;

        System.out.println("Using seed: " + ArrayUtils.toString(seed));
        System.out.printf("Generating %d random numbers:%n", numbersToGenerate);
        
        try {
            SmartCard.start();
            
            CardRequest cardRequest = new CardRequest(CardRequest.NEWCARD, null, null);

            CardTerminalRegistry ctr = CardTerminalRegistry.getRegistry();
            Enumeration terminals = ctr.getCardTerminals();
            ctr.remove((CardTerminal) terminals.nextElement());
            ctr.remove((CardTerminal) terminals.nextElement());
            
            SmartCard smartCard = SmartCard.waitForCard(cardRequest);
            
            PassThruCardService card = (PassThruCardService) smartCard.getCardService(PassThruCardService.class, true);
            
            ResponseAPDU response = card.sendCommandAPDU(selectRandomApplet);
            System.out.println("Selecting Random Applet: 0x" + Integer.toHexString(response.sw()));

            CommandAPDU command = new CommandAPDU(new byte[] {
                    RANDOM_CLA, SET_SEED_INS, P1, P2, (byte) (seed.length + 1),
                    EMPTY, EMPTY, EMPTY, EMPTY, EMPTY, EMPTY, EMPTY, EMPTY, EMPTY, EMPTY, 
                    EMPTY, EMPTY, EMPTY, EMPTY, EMPTY, EMPTY, EMPTY, EMPTY, EMPTY, EMPTY,
                    EMPTY, EMPTY, EMPTY, EMPTY, EMPTY, EMPTY, EMPTY, EMPTY, EMPTY, EMPTY});
            
            byte[] buffer = command.getBuffer();
            buffer[OFFSET_CDATA] = (byte) seed.length;
            
            System.arraycopy(seed, 0, buffer, OFFSET_CDATA + 1, seed.length);
            
            System.out.println("APDU buffer with seed is: " + ArrayUtils.toString(buffer));
            
            response = card.sendCommandAPDU(command);
            System.out.println("Sending seed response: 0x" + Integer.toHexString(response.sw()));
            
            for (int i = 1; i <= numbersToGenerate ; i++) {
                response = card.sendCommandAPDU(getNextRandomNumberAPDU);
                System.out.println("Getting next random number: 0x" + Integer.toHexString(response.sw()));
                byte[] data = response.data();
                
                System.out.println("Retrieved data: " + ArrayUtils.toString(data));
                
                if ((response.sw1() & 0xFF) == 0x90 && response.sw2() == 0x00) {
                    System.out.println("The random number is: " + data[0]);
                    
                    if (data[0] == 0) {
                        zeroCount++;
                    }
                }
            }
            
            System.out.println("Finished.");
            System.out.printf("Zeros: %.2f%%", zeroCount / (numbersToGenerate / 100f));
            
        } finally {
            try {
                SmartCard.shutdown();
                
            } catch(Exception e) {
                e.printStackTrace();
            }
        }
    }
}
