/*
 * @(#)ConfigurationParserTest.java
 *
 * $Revision: 1.3 $
 * $Date: 2006/04/06 19:10:12 $
 *
 * Copyright (c) 2005-2006 RWTH Aachen University, Lehr- und Forschungsgebiet Informatik 4.
 * All rights reserved.
 */

package de.rwth.informatik.lufgi4.trustedpals.gui;

import java.util.Arrays;

import junit.framework.TestCase;

public class ConfigurationParserTest extends TestCase {
    
    public void testParsing() throws Exception {
        byte[] data = new byte[] {3, 2, 1, 3, 4, 1, 1, 1, 1, 3, 7, 7, 7, 5, 2, 2, 2, 2, 2,
                12, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8};
        
        ConfigurationParser parser = new ConfigurationParser(data, true); 
        
        assertEquals(3, parser.getHostCount());
        assertEquals(2, parser.getHostID());
        assertEquals(1, parser.getSourceHost());
        assertEquals(3, parser.getDestinationHost());
        
        assertTrue(Arrays.equals(new byte[] {1, 1, 1, 1}, parser.getOwnItemDescription()));
        assertEquals("1111", parser.getOwnItemDescriptionAsString());

        assertTrue(Arrays.equals(new byte[] {7, 7, 7}, parser.getOwnItemContent()));
        assertEquals("777", parser.getOwnItemContentAsString());
        
        assertTrue(Arrays.equals(new byte[] {2, 2, 2, 2, 2}, parser.getWantedItemDescription()));
        assertEquals("22222", parser.getWantedItemDescriptionAsString());
        
        assertTrue(Arrays.equals(new byte[] {8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8}, parser.getSeed()));
        assertEquals("888888888888", parser.getSeedAsString());
    }
    
    public void testByteArrayToString() throws Exception {
        String result = null;
        
        result = ConfigurationParser.byteArrayToString(new byte[] {1, 2, 3});
        assertEquals("123", result);
        
        result = ConfigurationParser.byteArrayToString(new byte[] {5, 5, 5, 5, 5});
        assertEquals("55555", result);
        
        result = ConfigurationParser.byteArrayToString(new byte[] {});
        assertEquals("", result);
    }
}
