/*
 * @(#)MessageTest.java
 *
 * $Revision: 1.3 $
 * $Date: 2006/04/06 19:10:12 $
 *
 * Copyright (c) 2005-2006 RWTH Aachen University, Lehr- und Forschungsgebiet Informatik 4.
 * All rights reserved.
 */

package de.rwth.informatik.lufgi4.trustedpals.messaging;

import java.util.Arrays;

import de.rwth.informatik.lufgi4.trustedpals.protocol.Item;
import junit.framework.TestCase;

public class MessageTest extends TestCase {

    private byte[] content1;
    
    private byte[] content2;
    
    private byte[] content3;
    
    private byte[] content4;
    
    private byte[] content5;
    
    private Message m1, m2, m3, m4, m5;
    
    private Item i1, i2;
    
    private byte[] vote1, vote2;
    
    private byte[] locked1, locked2;
    
    protected void setUp() throws Exception {
        content1 = new byte[] {1, 2, 3, 7, 3, 1, 1, 1, 0, 0, 0};
        content2 = new byte[] {3, 2, 3, 7, 1, 1, 0, 1, 1, 1, 1};
        content3 = new byte[] {1, 3, 5, 0, 3, 2, 1, 1, 3, 2, 2, 2};
        content4 = new byte[] {4, 3, 9, 0, 0, 3, 1, 1, 1, 5, 2, 2, 2, 2, 2};
        content5 = new byte[] {1, 2, 3, 9, 5, 1};
        
        i1 = new Item(new byte[] {1, 1}, new byte[] {2, 2, 2});
        i2 = new Item(new byte[] {1, 1, 1}, new byte[] {2, 2, 2, 2, 2});
        
        vote1 = new byte[] {1, 1, 1};
        vote2 = new byte[] {1, 0, 1};
        
        locked1 = new byte[] {0, 0, 0};
        locked2 = new byte[] {1, 1, 1};
        
        m1 = new Message(content1);
        m2 = new Message(content2);
        m3 = new Message(content3);
        m4 = new Message(content4);
        m5 = new Message(content5);
    }

    public void testSender() {
        assertEquals(1, m1.getSender());
        assertEquals(3, m2.getSender());
        assertEquals(1, m3.getSender());
        assertEquals(4, m4.getSender());
        assertEquals(1, m5.getSender());
    }
    
    public void testReceiver() {
        assertEquals(2, m1.getReceiver());
        assertEquals(2, m2.getReceiver());
        assertEquals(3, m3.getReceiver());
        assertEquals(3, m4.getReceiver());
        assertEquals(2, m5.getReceiver());
    }
    
    public void testHostCount() {
        assertEquals(3, m1.getHostCount());
        assertEquals(3, m2.getHostCount());
        assertEquals(5, m3.getHostCount());
        assertEquals(9, m4.getHostCount());
        assertEquals(3, m5.getHostCount());
    }

    public void testRound() {
        assertEquals(7, m1.getRound());
        assertEquals(7, m2.getRound());
        assertEquals(0, m3.getRound());
        assertEquals(0, m4.getRound());
        assertEquals(9, m5.getRound());
    }
    
    public void testRandomNumber() {
        assertEquals(3, m3.getRandomNumber());
        assertEquals(0, m4.getRandomNumber());
        
        try {
            m1.getRandomNumber();
            fail();
            
        } catch (IllegalStateException e) {
            assertTrue(true);
        }
    }

    public void testItem() {
        assertEquals(i1, m3.getItem());
        assertEquals(i2, m4.getItem());
        
        try {
            m2.getItem();
            fail();
            
        } catch (IllegalStateException e) {
            assertTrue(true);
        }
    }
    
    public void testVote() {
        assertTrue(Arrays.equals(vote1, m1.getVote()));
        assertTrue(Arrays.equals(vote2, m2.getVote()));
        
        try {
            m3.getVote();
            fail();
            
        } catch (IllegalStateException e) {
            assertTrue(true);
        }
    }
    
    public void testLocked() {
        assertTrue(Arrays.equals(locked1, m1.getLocked()));
        assertTrue(Arrays.equals(locked2, m2.getLocked()));
        
        try {
            m4.getLocked();
            fail();
            
        } catch (IllegalStateException e) {
            assertTrue(true);
        }
    }
    
    public void testValue() {
        assertEquals(1, m5.getValue());
    }
    
    public void testFlags() {
        assertEquals(true, m1.isDummy());
        assertEquals(false, m1.isFake());

        assertEquals(false, m2.isDummy());
        assertEquals(false, m2.isFake());
        
        try {
            m4.isDummy();
            fail();
            
        } catch (IllegalStateException e) {
            assertTrue(true);
        }
    }
    
    public void testSize() {
        assertEquals(11, m1.getSize());
        assertEquals(11, m2.getSize());
        assertEquals(12, m3.getSize());
        assertEquals(15, m4.getSize());
        assertEquals(6, m5.getSize());
    }
    
    public void testEquals() {
        byte[] c1 = (byte[]) content1.clone();
        Message message1 = new Message(c1);
        
        assertTrue(message1.equals(m1));
        assertFalse(message1.equals(m2));
    }
}
