/*
 * @(#)GDFairExchangeProtocolTest.java
 *
 * $Revision: 1.2 $
 * $Date: 2006/04/06 19:10:12 $
 *
 * Copyright (c) 2005-2006 RWTH Aachen University, Lehr- und Forschungsgebiet Informatik 4.
 * All rights reserved.
 */

package de.rwth.informatik.lufgi4.trustedpals.protocol;

import java.util.ArrayList;
import java.util.List;

import de.rwth.informatik.lufgi4.trustedpals.messaging.Message;
import junit.framework.TestCase;

public class GDFairExchangeProtocolTest extends TestCase {

    private GDFairExchangeProtocol protocol = null;
    
    private byte[] content1;
    
    private byte[] content2;
    
    private byte[] content3;
    
    private byte[] content4;
    
    private byte[] content5;
    
    protected void setUp() throws Exception {
        protocol = new GDFairExchangeProtocol();
        
        content1 = new byte[] {1, 2, 3, 7, 1, 1, 1, 1, 0, 0, 0};
        content2 = new byte[] {3, 2, 3, 7, 1, 1, 1, 1, 1, 1, 1};
        content3 = new byte[] {1, 2, 5, 0, 3, 2, 1, 1, 3, 2, 2, 2};
        content4 = new byte[] {4, 2, 5, 0, 0, 3, 1, 1, 1, 5, 2, 2, 2, 2, 2};
        content5 = new byte[] {1, 2, 3, 0, 7, 0};
    }
    
    public void testGetMessages() {
        Message m1 = new Message(content1);
        Message m2  = new Message(content2);
        protocol.setHostCount((byte) 3);
        
        byte[] tempMessages = new byte[content1.length + content2.length];
        
        for (int i = 0; i < content1.length; i++) {
            tempMessages[i] = content1[i];
        }
        
        for (int i = 0; i < content2.length; i++) {
            tempMessages[content1.length + i] = content2[i];
        }
        
        protocol.setMessages(tempMessages);
        List messages = protocol.getMessages();
        
        assertEquals(2, messages.size());
        
        assertEquals(m1, messages.get(0));
        assertEquals(m2, messages.get(1));
        
        m1 = new Message(content3);
        m2  = new Message(content4);
        Message m3 = new Message(content5);
        
        tempMessages = new byte[content3.length + content4.length + content5.length];
        
        for (int i = 0; i < content3.length; i++) {
            tempMessages[i] = content3[i];
        }
        
        for (int i = 0; i < content4.length; i++) {
            tempMessages[content3.length + i] = content4[i];
        }

        for (int i = 0; i < content5.length; i++) {
            tempMessages[content3.length + content4.length + i] = content5[i];
        }
        
        protocol.setMessages(tempMessages);
        messages = protocol.getMessages();
        
        assertEquals(3, messages.size());
        
        assertEquals(m1, messages.get(0));
        assertEquals(m2, messages.get(1));
        assertEquals(m3, messages.get(2));
    }
    
    public void testPrepareFakeMessages() {
        protocol.setHostCount((byte) 3);
        protocol.setHostID((byte) 2);
        protocol.setRound((byte) 5);
        
        Message m1 = new Message(new byte[] {2, 1, 3, 5, 2, 0, 0, 0, 0, 0, 0});
        Message m2 = new Message(new byte[] {2, 3, 3, 5, 2, 0, 0, 0, 0, 0, 0});
        
        protocol.prepareFakeMessages();
        
        List messages = protocol.getMessages();
        
        assertEquals(2, messages.size());
        
        assertEquals(m1, messages.get(0));
        assertEquals(m2, messages.get(1));
    }
    
    public void testPrepareNormalMessages() {
        protocol.setHostCount((byte) 4);
        protocol.setHostID((byte) 3);
        protocol.setRound((byte) 7);
        protocol.setNewVote(new byte[] {1, 1, 0, 1});
        protocol.setLocked(new byte[] {0, 0, 1, 0});
        
        Message m1 = new Message(new byte[] {3, 1, 4, 7, 1, 1, 1, 0, 1, 0, 0, 1, 0});
        Message m2 = new Message(new byte[] {3, 2, 4, 7, 1, 1, 1, 0, 1, 0, 0, 1, 0});
        Message m3 = new Message(new byte[] {3, 4, 4, 7, 1, 1, 1, 0, 1, 0, 0, 1, 0});
        
        protocol.prepareNormalMessages();
        
        List messages = protocol.getMessages();
        
        assertEquals(3, messages.size());
        
        assertEquals(m1, messages.get(0));
        assertEquals(m2, messages.get(1));
        assertEquals(m3, messages.get(2));
    }
    
    public void testPrepareMessageWithItem() {
        protocol.setHostCount((byte) 3);
        protocol.setHostID((byte) 1);
        protocol.setDestinationHost((byte) 2);
        protocol.setSourceHost((byte) 3);
        protocol.setRound((byte) 0);
        protocol.setFakeRoundsCount((byte) 7);
        protocol.setOwnItem(new Item(new byte[] {3, 2, 1}, new byte[] {4, 4, 4, 4}));
        
        Message m = new Message(new byte[] {1, 2, 3, 0, 7, 3, 3, 2, 1, 4, 4, 4, 4, 4});
        
        int length = protocol.prepareMessageWithItem();
        
        assertEquals(14, length);
        
        List messages = protocol.getMessages();
        
        assertEquals(1, messages.size());
        
        assertEquals(m, messages.get(0));
    }

    public void testPrepareInitiatorMessages() {
        protocol.setHostCount((byte) 4);
        protocol.setHostID((byte) 3);
        protocol.setDestinationHost((byte) 4);
        protocol.setSourceHost((byte) 1);
        protocol.setRound((byte) 0);
        protocol.setFakeRoundsCount((byte) 13);
        protocol.setOwnItem(new Item(new byte[] {3, 2, 1}, new byte[] {4, 4, 4, 4}));
        
        Message m1 = new Message(new byte[] {3, 4, 4, 0, 13, 3, 3, 2, 1, 4, 4, 4, 4, 4});
        Message m2 = new Message(new byte[] {3, 1, 4, 0, 13, 0});
        Message m3 = new Message(new byte[] {3, 2, 4, 0, 13, 0});
        
        protocol.prepareInitiatorMessages();
        
        List messages = protocol.getMessages();
        
        assertEquals(3, messages.size());
        
        assertEquals(m1, messages.get(0));
        assertEquals(m2, messages.get(1));
        assertEquals(m3, messages.get(2));
    }
    
    public void testMessagesToBytesConversion() {
        List messages = new ArrayList();
        
        Message m1 = new Message(content1);
        Message m2  = new Message(content2);

        messages.add(m1);
        messages.add(m2);
        
        byte[] tempMessages = new byte[content1.length + content2.length];
        
        for (int i = 0; i < content1.length; i++) {
            tempMessages[i] = content1[i];
        }
        
        for (int i = 0; i < content2.length; i++) {
            tempMessages[content1.length + i] = content2[i];
        }
        
        byte[] data = protocol.messagesToBytes(messages);
        
        for (int i = 0; i < tempMessages.length; i++) {
            assertEquals(tempMessages[i], data[i]);    
        }
        
        for (int i = tempMessages.length; i < data.length; i++) {
            assertEquals(0, data[i]);    
        }
    }

    public void testHasReceivedItemFromSourceHost() {
        protocol.setHostCount((byte) 3);
        protocol.setHostID((byte) 3);
        protocol.setSourceHost((byte) 2);

        protocol.setData(new byte[] {0, 0, 0, 0, 0, 0});
        assertFalse(protocol.hasReceivedItemFromSourceHost());
        
        protocol.setData(new byte[] {1, 3, 3, 0, 7, 0});
        assertFalse(protocol.hasReceivedItemFromSourceHost());

        protocol.setData(new byte[] {2, 3, 3, 0, 0, 0});
        assertFalse(protocol.hasReceivedItemFromSourceHost());
        
        protocol.setData(new byte[] {2, 3, 3, 0, 0, 0, 1, 3, 3, 0, 17, 0});
        assertFalse(protocol.hasReceivedItemFromSourceHost());
        
        protocol.setData(new byte[] {2, 3, 3, 0, 0, 2, 1, 1, 4, 2, 2, 2, 2});
        assertTrue(protocol.hasReceivedItemFromSourceHost());
        assertEquals(2, protocol.getReceivedItemDescriptionLength());
        assertEquals(1, protocol.getReceivedItemDescription()[0]);
        assertEquals(1, protocol.getReceivedItemDescription()[1]);
        assertEquals(0, protocol.getReceivedItemDescription()[2]);
        assertEquals(4, protocol.getReceivedItemContentLength());
        assertEquals(2, protocol.getReceivedItemContent()[0]);
        assertEquals(2, protocol.getReceivedItemContent()[1]);
        assertEquals(2, protocol.getReceivedItemContent()[2]);
        assertEquals(2, protocol.getReceivedItemContent()[3]);
        assertEquals(0, protocol.getReceivedItemContent()[4]);
        
        protocol.setData(new byte[] {2, 3, 3, 0, 0, 2, 1, 1, 4, 2, 2, 2, 2, 1, 3, 3, 0, 17, 0});
        assertTrue(protocol.hasReceivedItemFromSourceHost());
        assertEquals(2, protocol.getReceivedItemDescriptionLength());
        assertEquals(1, protocol.getReceivedItemDescription()[0]);
        assertEquals(1, protocol.getReceivedItemDescription()[1]);
        assertEquals(0, protocol.getReceivedItemDescription()[2]);
        assertEquals(4, protocol.getReceivedItemContentLength());
        assertEquals(2, protocol.getReceivedItemContent()[0]);
        assertEquals(2, protocol.getReceivedItemContent()[1]);
        assertEquals(2, protocol.getReceivedItemContent()[2]);
        assertEquals(2, protocol.getReceivedItemContent()[3]);
        assertEquals(0, protocol.getReceivedItemContent()[4]);
        
        protocol.setData(new byte[] {1, 3, 3, 0, 17, 0, 2, 3, 3, 0, 0, 2, 1, 1, 4, 2, 2, 2, 2});
        assertTrue(protocol.hasReceivedItemFromSourceHost());
        assertEquals(2, protocol.getReceivedItemDescriptionLength());
        assertEquals(1, protocol.getReceivedItemDescription()[0]);
        assertEquals(1, protocol.getReceivedItemDescription()[1]);
        assertEquals(0, protocol.getReceivedItemDescription()[2]);
        assertEquals(4, protocol.getReceivedItemContentLength());
        assertEquals(2, protocol.getReceivedItemContent()[0]);
        assertEquals(2, protocol.getReceivedItemContent()[1]);
        assertEquals(2, protocol.getReceivedItemContent()[2]);
        assertEquals(2, protocol.getReceivedItemContent()[3]);
        assertEquals(0, protocol.getReceivedItemContent()[4]);
    }
    
    public void testHasReceivedNormalMessageFromHost() {
        protocol.setHostCount((byte) 3);
        protocol.setHostID((byte) 3);
        protocol.setRound((byte) 7);
        
        protocol.setData(new byte[] {0, 0, 0, 0, 0, 0});
        assertFalse(protocol.hasReceivedNormalMessageFromHost((byte) 2));
        
        // dummy message
        protocol.setData(new byte[] {1, 3, 3, 7, 3, 0, 0, 0, 1, 1, 1});
        assertFalse(protocol.hasReceivedNormalMessageFromHost((byte) 1));
        assertFalse(protocol.hasReceivedNormalMessageFromHost((byte) 2));

        // normal message
        protocol.setData(new byte[] {1, 3, 3, 7, 1, 0, 1, 0, 1, 1, 0});
        assertTrue(protocol.hasReceivedNormalMessageFromHost((byte) 1));

        assertEquals(0, protocol.getReceivedVote()[0]);
        assertEquals(1, protocol.getReceivedVote()[1]);
        assertEquals(0, protocol.getReceivedVote()[2]);
        assertEquals(0, protocol.getReceivedVote()[3]);
        assertEquals(0, protocol.getReceivedVote()[4]);
        
        assertEquals(1, protocol.getReceivedLocked()[0]);
        assertEquals(1, protocol.getReceivedLocked()[1]);
        assertEquals(0, protocol.getReceivedLocked()[2]);
        assertEquals(0, protocol.getReceivedLocked()[3]);
        assertEquals(0, protocol.getReceivedLocked()[4]);
        
        assertFalse(protocol.hasReceivedNormalMessageFromHost((byte) 2));
        assertFalse(protocol.hasReceivedNormalMessageFromHost((byte) 8));
        
        // first is dummy, second is normal
        protocol.setData(new byte[] {1, 3, 3, 7, 3, 0, 0, 0, 1, 1, 1, 2, 3, 3, 7, 1, 0, 0, 0, 1, 1, 1});
        assertFalse(protocol.hasReceivedNormalMessageFromHost((byte) 1));
        assertTrue(protocol.hasReceivedNormalMessageFromHost((byte) 2));

        assertEquals(0, protocol.getReceivedVote()[0]);
        assertEquals(0, protocol.getReceivedVote()[1]);
        assertEquals(0, protocol.getReceivedVote()[2]);
        assertEquals(0, protocol.getReceivedVote()[3]);
        assertEquals(0, protocol.getReceivedVote()[4]);
        
        assertEquals(1, protocol.getReceivedLocked()[0]);
        assertEquals(1, protocol.getReceivedLocked()[1]);
        assertEquals(1, protocol.getReceivedLocked()[2]);
        assertEquals(0, protocol.getReceivedLocked()[3]);
        assertEquals(0, protocol.getReceivedLocked()[4]);
        
        // first is dummy, second is normal, empty space at the end
        protocol.setData(new byte[] {1, 3, 3, 7, 3, 0, 0, 0, 1, 1, 1, 2, 3, 3, 7, 1, 0, 0, 0, 1, 1, 1, 0, 0, 0, 0});
        assertFalse(protocol.hasReceivedNormalMessageFromHost((byte) 1));
        assertTrue(protocol.hasReceivedNormalMessageFromHost((byte) 2));
        
        assertEquals(0, protocol.getReceivedVote()[0]);
        assertEquals(0, protocol.getReceivedVote()[1]);
        assertEquals(0, protocol.getReceivedVote()[2]);
        assertEquals(0, protocol.getReceivedVote()[3]);
        assertEquals(0, protocol.getReceivedVote()[4]);
        
        assertEquals(1, protocol.getReceivedLocked()[0]);
        assertEquals(1, protocol.getReceivedLocked()[1]);
        assertEquals(1, protocol.getReceivedLocked()[2]);
        assertEquals(0, protocol.getReceivedLocked()[3]);
        assertEquals(0, protocol.getReceivedLocked()[4]);

        // first is normal, second is dummy
        protocol.setData(new byte[] {1, 3, 3, 7, 1, 1, 0, 1, 1, 0, 1, 2, 3, 3, 7, 3, 0, 0, 0, 1, 1, 1});
        assertTrue(protocol.hasReceivedNormalMessageFromHost((byte) 1));
        
        assertEquals(1, protocol.getReceivedVote()[0]);
        assertEquals(0, protocol.getReceivedVote()[1]);
        assertEquals(1, protocol.getReceivedVote()[2]);
        assertEquals(0, protocol.getReceivedVote()[3]);
        assertEquals(0, protocol.getReceivedVote()[4]);
        
        assertEquals(1, protocol.getReceivedLocked()[0]);
        assertEquals(0, protocol.getReceivedLocked()[1]);
        assertEquals(1, protocol.getReceivedLocked()[2]);
        assertEquals(0, protocol.getReceivedLocked()[3]);
        assertEquals(0, protocol.getReceivedLocked()[4]);
        
        assertFalse(protocol.hasReceivedNormalMessageFromHost((byte) 2));
    }
    
    public void testIsReceivedItemCorrect() {
        protocol.setWantedItemDescription(new byte[] {1, 2, 3, 4});
        
        protocol.setReceivedItemDescriptionLength((byte) 4);
        protocol.setReceivedItemDescription(new byte[] {1, 2, 3, 4});
        assertTrue(protocol.isReceivedItemCorrect());
        
        protocol.setReceivedItemDescriptionLength((byte) 4);
        protocol.setReceivedItemDescription(new byte[] {1, 2, 3, 4, 0, 0, 0});
        assertTrue(protocol.isReceivedItemCorrect());

        protocol.setReceivedItemDescriptionLength((byte) 4);
        protocol.setReceivedItemDescription(new byte[] {1, 2, 3, 4, 5, 0, 0});
        assertFalse(protocol.isReceivedItemCorrect());
        
        protocol.setReceivedItemDescriptionLength((byte) 3);
        protocol.setReceivedItemDescription(new byte[] {1, 2, 3, 0, 0, 0});
        assertFalse(protocol.isReceivedItemCorrect());

        protocol.setReceivedItemDescriptionLength((byte) 5);
        protocol.setReceivedItemDescription(new byte[] {1, 2, 3, 4, 5, 0, 0, 0});
        assertFalse(protocol.isReceivedItemCorrect());

        protocol.setReceivedItemDescriptionLength((byte) 5);
        protocol.setReceivedItemDescription(new byte[] {1, 2, 3, 4, 0, 0, 0, 0});
        assertFalse(protocol.isReceivedItemCorrect());
    }
    
    public void testHasReceivedRandomNumberFromInitiator() {
        protocol.setHostCount((byte) 3);
        protocol.setHostID((byte) 3);

        protocol.setData(new byte[] {2, 3, 3, 0, 0, 0});
        assertFalse(protocol.hasReceivedRandomNumberFromInitiator());

        protocol.setData(new byte[] {1, 3, 3, 0, 13, 0});
        assertTrue(protocol.hasReceivedRandomNumberFromInitiator());
        assertEquals(13, protocol.getFakeRoundsCount());
        
        protocol.setData(new byte[] {2, 3, 3, 0, 0, 0, 1, 3, 3, 0, 17, 0});
        assertTrue(protocol.hasReceivedRandomNumberFromInitiator());
        assertEquals(17, protocol.getFakeRoundsCount());
        
        protocol.setData(new byte[] {2, 3, 3, 0, 0, 2, 1, 1, 4, 2, 2, 2, 2});
        assertFalse(protocol.hasReceivedRandomNumberFromInitiator());
        
        protocol.setData(new byte[] {2, 3, 3, 0, 0, 2, 1, 1, 4, 2, 2, 2, 2, 1, 3, 3, 0, 21, 0});
        assertTrue(protocol.hasReceivedRandomNumberFromInitiator());
        assertEquals(21, protocol.getFakeRoundsCount());
    }
    
    public void testHasReceivedFakeMessagesFromAllHosts() {
        Message m1 = new Message(new byte[] {1, 2, 3, 7, 2, 0, 0, 0, 0, 0, 0});
        Message m2 = new Message(new byte[] {3, 2, 3, 7, 2, 0, 0, 0, 0, 0, 0});

        protocol.setHostCount((byte) 3);
        protocol.setHostID((byte) 2);
        protocol.setRound((byte) 7);
        protocol.setFakeRoundsCount((byte) 13);
        
        List messages = new ArrayList();
        
        messages.add(m1);
        protocol.messagesToBytes(messages);
        assertFalse(protocol.hasReceivedFakeMessagesFromAllHosts());

        messages.add(m2);
        protocol.messagesToBytes(messages);
        assertTrue(protocol.hasReceivedFakeMessagesFromAllHosts());

        messages.remove(0);
        protocol.messagesToBytes(messages);
        assertFalse(protocol.hasReceivedFakeMessagesFromAllHosts());
    }
    
    public void testInitiatorDetermination() {
        protocol.setHostCount((byte) 4);

        protocol.setHostID((byte) 1);
        assertTrue(protocol.isInitiator());
        
        protocol.setHostID((byte) 3);
        assertFalse(protocol.isInitiator());
    }
    
    public void testRandomNumberSelection() {
        int k = protocol.pickRandomNumber();
        assertTrue(k >= 1 && k <= 4);
        
        k = protocol.pickRandomNumber();
        assertTrue(k >= 1 && k <= 4);

        k = protocol.pickRandomNumber();
        assertTrue(k >= 1 && k <= 4);

        k = protocol.pickRandomNumber();
        assertTrue(k >= 1 && k <= 4);
    }
    
    public void testMaxFaultyProcessesComputation() {
        protocol.setHostCount((byte) 3);
        assertEquals(1, protocol.computeMaxFaultyProcesses());

        protocol.setHostCount((byte) 4);
        assertEquals(1, protocol.computeMaxFaultyProcesses());
        
        protocol.setHostCount((byte) 5);
        assertEquals(2, protocol.computeMaxFaultyProcesses());
        
        protocol.setHostCount((byte) 6);
        assertEquals(2, protocol.computeMaxFaultyProcesses());
    }
}
