/*
 * @(#)RandomTest.java
 *
 * $Revision: 1.2 $
 * $Date: 2006/04/06 19:10:12 $
 *
 * Copyright (c) 2005-2006 RWTH Aachen University, Lehr- und Forschungsgebiet Informatik 4.
 * All rights reserved.
 */

package de.rwth.informatik.lufgi4.trustedpals.random;

import de.rwth.informatik.lufgi4.trustedpals.random.Random;
import junit.framework.TestCase;

/**
 * 
 * 
 * @author Milan Fort
 * @version $Revision: 1.2 $, $Date: 2006/04/06 19:10:12 $
 * 
 * @since 1.0
 */
public class RandomTest extends TestCase {
    
    Random random = null, random2 = null;
    
    protected void setUp() throws Exception {
        random = new Random();
        random2 = new Random();
    }

    public void testSetSeed() throws Exception {
        short seedLength = random.getSeedLength();
        assertEquals(0, seedLength);
        
        byte[] testSeed = new byte[] {1, 2, 3, 4, 5, 6, 7, 8, 9};

        random.setSeed(testSeed, (short) 0, (short) 9);
        assertEquals(9, random.getSeedLength());
        byte[] currentSeed = random.getSeed(); 
        
        for (int i = 0; i < 9; i++) {
            assertEquals(testSeed[i], currentSeed[i]);
        }
        
        random.setSeed(testSeed, (short) 2, (short) 3);
        assertEquals(3, random.getSeedLength());
        currentSeed = random.getSeed();        
        
        for (int i = 0; i < 3; i++) {
            assertEquals(testSeed[i+2], currentSeed[i]);
        }
    }
    
    public void testComputeOnesCount() throws Exception {
        assertEquals((byte) 0, Random.computeOnesCount((byte) 0));
        assertEquals((byte) 1, Random.computeOnesCount((byte) 1));
        assertEquals((byte) 2, Random.computeOnesCount((byte) 3));
        assertEquals((byte) 1, Random.computeOnesCount((byte) 4));
        assertEquals((byte) 2, Random.computeOnesCount((byte) 5));
        assertEquals((byte) 2, Random.computeOnesCount((byte) 6));
        assertEquals((byte) 3, Random.computeOnesCount((byte) 7));
        assertEquals((byte) 1, Random.computeOnesCount((byte) 8));
        
        assertEquals((byte) 3, Random.computeOnesCount((byte) 13));
        
        assertEquals((byte) 5, Random.computeOnesCount((byte) 62));
        assertEquals((byte) 6, Random.computeOnesCount((byte) 63));
        
        assertEquals((byte) 4, Random.computeOnesCount((byte) 0xF0));
        assertEquals((byte) 5, Random.computeOnesCount((byte) 0xF8));
        assertEquals((byte) 8, Random.computeOnesCount((byte) 0xFF));
    }
    
    public void testHasOddOnesCount() throws Exception {
       assertFalse(Random.hasOddOnesCount(new byte[] {1, 1}));
       assertTrue(Random.hasOddOnesCount(new byte[] {1, 1, 1}));
       assertFalse(Random.hasOddOnesCount(new byte[] {1, 1, 1, 1}));
       
       
       assertTrue(Random.hasOddOnesCount(new byte[] {62, 7, 1}));
       assertFalse(Random.hasOddOnesCount(new byte[] {1, 1, 1, 1, 6, (byte) 0xFF}));
    }
    
    public void testIncreaseByte() throws Exception {
        assertEquals(7, Random.increaseByte((byte) 6));
        assertEquals(100, Random.increaseByte((byte) 99));
        assertEquals(127, Random.increaseByte((byte) 126));
        assertEquals(-128, Random.increaseByte((byte) 127));
        assertEquals(-127, Random.increaseByte((byte) -128));
        assertEquals(-12, Random.increaseByte((byte) -13));
    }
    
    public void testIncreaseSeedByteToUpdate() throws Exception {
        short seedLength = random.getSeedLength();
        assertEquals(0, seedLength);
        
        byte[] testSeed = new byte[] {0, 1, 2, 3, 4};

        random.setSeed(testSeed, (short) 0, (short) 5);
        assertEquals(5, random.getSeedLength());
        
        assertEquals(0, random.getSeedByteToUpdate());
        
        random.increaseSeedByteToUpdate();
        assertEquals(1, random.getSeedByteToUpdate());
        
        random.increaseSeedByteToUpdate();
        assertEquals(2, random.getSeedByteToUpdate());
        
        random.increaseSeedByteToUpdate();
        assertEquals(3, random.getSeedByteToUpdate());
        
        random.increaseSeedByteToUpdate();
        assertEquals(4, random.getSeedByteToUpdate());
        
        random.increaseSeedByteToUpdate();
        assertEquals(0, random.getSeedByteToUpdate());
     
        random.increaseSeedByteToUpdate();
        assertEquals(1, random.getSeedByteToUpdate());
    }
    
    public void testNextRandomNumber() throws Exception {
        random.setSeed( new byte[] {1, 22, 43, 45, 5, 76}, (short) 0, (short) 6);
        random2.setSeed(new byte[] {1, 22, 43, 45, 5, 76}, (short) 0, (short) 6);
        
        for (int i = 0; i < 1000; i++) {
            assertEquals(random.nextRandomNumber(), random2.nextRandomNumber());
        }
    }
}
